/*************************************************************************************************************************
5. Create Cohort Codes File
**************************************************************************************************************************/

%let basename = C:/Public_Training_Materials/Lab;	

libname in "&basename./codes";
libname out "&basename./public_mpl1r_wp001_synpufs_v01/inputfiles";
%let wpnum = wp001 ;  
options mlogic mprint symbolgen;
ods listing close;

/*SPECIFY EXPOSURE INDEX-DEFINING CODES*/
%macro indexcodes (title,group,codes);
	data indexcodes&title.;
	format group $40. stockgroup $30. codecat $2. codetype $3. code1 $11. code $11. caresettingprincipal $41. 
	       t1_index $3. t2_index $3. t2_fup $3. conc_fup $3. t3_index $3. t3_fup $3. t4_index $3. t4_fup $3. t5_index $3. t6_index $3.
		   productapprovaldate Date9. productmarketingdate Date9. otherproductdate Date9. excludesupply $1. codesupply 8.
           rawlabdatetype $3. rawlabresult $3. ;
	set in.&codes.;
	group = "&group.";
	stockgroup = compress (descrip, ', ./\_-<>=;&[](){}%');
	codecat = CodeCat1;
	codetype = CodeType1; 
	code = compress (code1, ".");
	caresettingprincipal = " ";

	t1_index = 'NOT';
	t2_index = 'DEF'; /*When defining index-defining codes for exposure, t2_index must = 'DEF' */
	t2_fup = 'NOT';   /*When defining exposure, t2_fup must = 'NOT' */
	conc_fup = 'NOT';
	t3_index = 'NOT';
	t3_fup = 'NOT';
	t4_index = 'NOT';
	t4_fup = 'NOT';
	t5_index = 'NOT';
	t6_index = 'NOT';	
	productapprovaldate = .;
	productmarketingdate = .;
	otherproductdate = .;

	excludesupply = "N";
	codesupply = . ;
	rawlabdatetype = ""; 
	rawlabresult = ""; 
	drop code1 codecat1 codetype1 descrip;
	run;
%mend;

%indexcodes (1,typ_is, typical_antipsychotics);    /*Fill in group 1 name and code list to define index exposure */
%indexcodes (2,typ_ich, typical_antipsychotics);   /*Fill in group 2 name and code list to define index exposure */
%indexcodes (3,atyp_is, atypical_antipsychotics);  /*Fill in group 3 name and code list to define index exposure */
%indexcodes (4,atyp_ich, atypical_antipsychotics); /*Fill in group 4 name and code list to define index exposure */

%macro exportindexcodes(start,end);
	data index;
		set %do i=&start. %to &end.;
				indexcodes&i.
			%end;
		;
	run;
%mend;
%exportindexcodes (1,4);



/*SPECIFY EXPOSURE INCIDENCE AND TRUNCATION CODES*/
%macro inc_trunc_codes (title,group,codes);
	data inc_trunc_codes&title.;
	format group $40. stockgroup $30. codecat $2. codetype $3. code1 $11. code $11. caresettingprincipal $41. 
	       t1_index $3. t2_index $3. t2_fup $3. conc_fup $3. t3_index $3. t3_fup $3. t4_index $3. t4_fup $3. t5_index $3. t6_index $3.
		   productapprovaldate Date9. productmarketingdate Date9. otherproductdate Date9. excludesupply $1. codesupply 8.
           rawlabdatetype $3. rawlabresult $3. ;
	set in.&codes.;
	group = "&group.";
	stockgroup = compress (descrip, ', ./\_-<>=;&[](){}%');
	codecat = CodeCat1;
	codetype = CodeType1; 
	code = compress (code1, ".");
	caresettingprincipal = " ";

	t1_index = 'NOT';
	t2_index = 'IOT'; /*When defining incidence and truncation criteria for exposure, t2_index should = 'IOT' */
	t2_fup = 'NOT';   /*When defining exposure, t2_fup must = 'NOT' */
	conc_fup = 'NOT';
	t3_index = 'NOT';
	t3_fup = 'NOT';
	t4_index = 'NOT';
	t4_fup = 'NOT';
	t5_index = 'NOT';
	t6_index = 'NOT';	
	productapprovaldate = .;
	productmarketingdate = .;
	otherproductdate = .;

	excludesupply = "N";
	codesupply = . ;
	rawlabdatetype = ""; 
	rawlabresult = ""; 
	drop code1 codecat1 codetype1 descrip;
	run;
%mend;

%inc_trunc_codes (1,typ_is, atypical_antipsychotics);  /*Fill in group 1 name and code list to define incidence and truncation criteria */
%inc_trunc_codes (2,typ_ich, atypical_antipsychotics); /*Fill in group 2 name and code list to define incidence and truncation criteria */
%inc_trunc_codes (3,atyp_is, typical_antipsychotics);  /*Fill in group 3 name and code list to define incidence and truncation criteria */
%inc_trunc_codes (4,atyp_ich, typical_antipsychotics); /*Fill in group 4 name and code list to define incidence and truncation criteria */

%macro exportinc_trunc_codes(start,end);
	data incidence;
		set %do i=&start. %to &end.;
				inc_trunc_codes&i.
			%end;
		;
	run;
%mend;
%exportinc_trunc_codes (1,4);



/*SPECIFY OUTCOME-DEFINING CODES*/
%macro outcomecodes (title,group,codes);
	data outcomecodes&title.;
	format group $40. stockgroup $30. codecat $2. codetype $3. code1 $11. code $11. caresettingprincipal $41. 
	       t1_index $3. t2_index $3. t2_fup $3. conc_fup $3. t3_index $3. t3_fup $3. t4_index $3. t4_fup $3. t5_index $3. t6_index $3.
		   productapprovaldate Date9. productmarketingdate Date9. otherproductdate Date9. excludesupply $1. codesupply 8.
           rawlabdatetype $3. rawlabresult $3. ;
	set in.&codes.;
	group = "&group.";
	stockgroup = compress (descrip, ', ./\_-<>=;&[](){}%');
	codecat = CodeCat1;
	codetype = CodeType1; 
	code = compress (code1, ".");
	caresettingprincipal = "'IPP'";

	t1_index = 'NOT';
	t2_index = 'NOT'; /*When defining outcome, t2_index must = 'NOT' */
	t2_fup = 'DEF';   /*When defining outcome, t2_fup must = 'DEF' */
	conc_fup = 'NOT';
	t3_index = 'NOT';
	t3_fup = 'NOT';
	t4_index = 'NOT';
	t4_fup = 'NOT';
	t5_index = 'NOT';
	t6_index = 'NOT';	
	productapprovaldate = .;
	productmarketingdate = .;
	otherproductdate = .;

	excludesupply = "N";
	codesupply = . ;
	rawlabdatetype = ""; 
	rawlabresult = ""; 
	drop code1 codecat1 codetype1 descrip;
	run;
%mend;

%outcomecodes (1,typ_is, ischemic_stroke); 			 /*Fill in group 1 name and code list to define outcome */
%outcomecodes (2,typ_ich, intracranial_hemorrhage);  /*Fill in group 2 name and code list to define outcome */
%outcomecodes (3,atyp_is, ischemic_stroke); 		 /*Fill in group 3 name and code list to define outcome */
%outcomecodes (4,atyp_ich, intracranial_hemorrhage); /*Fill in group 4 name and code list to define outcome */

%macro exportoutcomecodes(start,end);
	data outcome;
		set %do i=&start. %to &end.;
				outcomecodes&i.
			%end;
		;
	run;
%mend;
%exportoutcomecodes (1,4);



/*SPECIFY OUTCOME INCIDENCE-DEFINING CODES*/
%macro outcomeinccodes (title,group,codes);
	data outcomeinccodes&title.;
	format group $40. stockgroup $30. codecat $2. codetype $3. code1 $11. code $11. caresettingprincipal $41. 
	       t1_index $3. t2_index $3. t2_fup $3. conc_fup $3. t3_index $3. t3_fup $3. t4_index $3. t4_fup $3. t5_index $3. t6_index $3.
		   productapprovaldate Date9. productmarketingdate Date9. otherproductdate Date9. excludesupply $1. codesupply 8.
           rawlabdatetype $3. rawlabresult $3. ;
	set in.&codes.;
	group = "&group.";
	stockgroup = compress (descrip, ', ./\_-<>=;&[](){}%');
	codecat = CodeCat1;
	codetype = CodeType1; 
	code = compress (code1, ".");
	caresettingprincipal = " ";

	t1_index = 'NOT';
	t2_index = 'NOT'; /*When defining outcome incidence, t2_index must = 'NOT' */
	t2_fup = 'IOC';   /*When defining outcome incidence, t2_fup must = 'IOC' */
	conc_fup = 'NOT';
	t3_index = 'NOT';
	t3_fup = 'NOT';
	t4_index = 'NOT';
	t4_fup = 'NOT';
	t5_index = 'NOT';
	t6_index = 'NOT';	
	productapprovaldate = .;
	productmarketingdate = .;
	otherproductdate = .;

	excludesupply = "N";
	codesupply = . ;
	rawlabdatetype = ""; 
	rawlabresult = ""; 
	drop code1 codecat1 codetype1 descrip;
	run;
%mend;

%outcomeinccodes (1,typ_is,intracranial_hemorrhage);  /*Fill in group 1 name and code list to define outcome incidence*/
%outcomeinccodes (2,typ_ich,ischemic_stroke);  		  /*Fill in group 2 name and code list to define outcome incidence */
%outcomeinccodes (3,atyp_is,intracranial_hemorrhage); /*Fill in group 3 name and code list to define outcome incidence */
%outcomeinccodes (4,atyp_ich,ischemic_stroke); 		  /*Fill in group 4 name and code list to define outcome incidence*/

%macro exportoutcomeinccodes(start,end);
	data outcomeinc;
		set %do i=&start. %to &end.;
				outcomeinccodes&i.
			%end;
		;
	run;
%mend;
%exportoutcomeinccodes (1,4);



data out.&wpnum._cohortcodes;
set index incidence outcome outcomeinc;
run;
