****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_assigncalmacrovars.sas  
*
* Created (mm/dd/yyyy): 06/30/2019
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This macro creates calendar based macro variables for reporting                              
*   
*  Program inputs:    
*	- date1: Start of period
*	- date2: End of period
* 
*  Program outputs:                                                                                                                                       
* 
*  PARAMETERS:                                                                       
*
* 
*  Programming Notes:                                                                                
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_assigncalmacrovars(date1=, date2=);

%put =====> MACRO CALLED: ms_assigncalmacrovars v1.0;

    data _null_;
		format startdate enddate date9.;
		    startdate=intnx('month',&date1.,0,'begin');  *in case not at the start of month;
		    enddate=intnx('month',&date2.,1,'begin')-1; *in case not at the end of month;
		    nummonths=intck('month',startdate,enddate)+1;
		    numyears=intck('year',startdate,enddate)+1;

		    call symputx("StartYear",year(startdate));
		    call symputx("StartMonth",month(startdate));
		    call symputx("EndYear",year(enddate));
		    call symputx("EndMonth",month(enddate));
		    call symputx("nummonths",nummonths);
		    call symputx("numyears",numyears);
	run;
    %put &StartYear. &StartMonth. &EndYear. &EndMonth. &nummonths. &numyears.;

	*PER and YEAR macro vars are global;
	%do int = 1 %to %eval(&nummonths.+1);
		%global per&int.;
	%end;
	%do int = 1 %to %eval(&numyears.+1);
		%global year&int.;
	%end;

	   data _null_;
	    do i=1 to &nummonths.+1;
	        call symput("PER"||strip(put(i,best.)),put(intnx('month',&date1.,i-1,'begin'),best.));
	    end;
	    do i=1 to &numyears.+1;
	        call symput("YEAR"||strip(put(i,best.)),put(intnx('year',&date1.,i-1,'begin'),best.));
	   	end;
    run;

%put NOTE: ******** END OF MACRO: ms_assigncalmacrovars v1.0 ********;

%mend ms_assigncalmacrovars;
