***************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidanum.sas  
*
* Created (mm/dd/yyyy): 12/19/2014
* Last modified: 06/30/2019
* Version: 4.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will compute the numerators for each cohort and call many other macros:
*	- creating the denominators 
*	- creating output tables
*	- performing multiple cohort analyses (PSA, MFM, Multiple Events, Concomitant Use, Overlap, Tree Extraction) 
*   
*  Program inputs:   
*	-infolder.&MONITORINGFILE.
*	-infolder.&USERSTRATA.
*	-infolder.&COMBOFILE.
*	-infolder.&COHORTFILE.
*	-infolder.&COHORTCODES.
*	-infolder.&INCLUSIONCODES. 
*	-infolder.&STOCKPILINGFILE.
*	-infolder.&ZIPFILE.
*	-infolder.&TYPE1FILE.
*	-infolder.&TYPE2FILE.
*	-infolder.&TYPE3FILE.
*	-infolder.&TYPE4FILE.
*	-infolder.&PREGDUR. 
*	-infolder.&TYPE5FILE.
*	-infolder.&TYPE6FILE. 
*	-infolder.&METADATAFILE.
*	-infolder.&COVARIATECODES. 
*	-infolder.&UTILFILE. 
*	-infolder.&COMORBFILE.
*	-infolder.&DRUGCLASSFILE.
*	-infolder.&MFUFILE.
*	-infolder.&MULTEVENTFILE. 
*	-infolder.&MULTEVENTFILE_ADHERE. 
*	-infolder.&CONCFILE. 
*	-infolder.&OVERLAPFILE. 
*	-infolder.&OVERLAPFILE_ADHERE. 
*	-infolder.&ITSFILE. 
*	-infolder.&TREATMENTPATHWAYS. 
*	-infolder.&MICOHORTFILE.
*	-infolder.&TREEFILE.
*	-infolder.&TREELOOKUP.
*	-infolder.&ICD10ICD9MAP.
*	-infolder.&COMPARISON.
*	-infolder.&ANALYTICSUBGROUPS.
*
*                                     
*  Program outputs:       
*	-Numerous dplocal and msoc datasets
*                                                                                                  
*  PARAMETERS:  
*
*	-RUNID					= Run identifier to denote each execution of the program
*	-PROJID					= SOC-defined value  
*	-WPTYPE					= SOC-defined value
*	-WPID					= SOC-defined value
*	-DPID					= SOC-defined value
*	-VERID					= SOC-defined value
*	-PERIODIDSTART			= Starting PERIODID from the MONITORINGFILE
*	-PERIODIDEND			= Ending PERIODID from the MONITORINGFILE
*	-ANALYSIS				= Indicates whether to perform tree extraction
*	-MONITORNGFILE			= Name of the SAS dataset that defines the relevant query periods for execution
*	-USERSTRATA				= Name of the SAS dataset that defines the stratifications to return in the MSOC folder
*	-COMBOFILE				= Name of the SAS dataset that will be used to create combination items. 
*						      When this parameter is specified, "CODES" input files is required, and "STOCK" is optional
*	-COHORTFILE				= Name of the SAS dataset that defines how the cohort should be created (gaps, coverage,...)
*	-COHORTCODES			= Name of the SAS dataset that includes the codes defining the cohort 
*	-INCLUSIONCODES			= Name of the SAS dataset that includes the codes defining inclusions/exclusions
*	-STOCKPILINGFILE		= Name of the SAS dataset that defines how the stockpiling algorithm should be performed
*	-RUN_ENVELOPE   		= Indicates if the envelope macro should include Adate or be turned off entirely
*	-FREEZEDATA				= Indicates if SDD patient data should be copied to DPLocal
*	-ZIPFILE				= Name of the SAS dataset that contains list of valid zip to geographical area mappings
*	-LABSCODEMAP			= Name of the SAS dataset that contains laboratory matching information
*	-DISTINDEX				= Indicates if distribution of index defining codes should be output to MSOC folder
*	-TYPE1FILE				= Name of the SAS dataset that defines type 1 cohort parameters (washouts, cohortdef, ...)
*	-TYPE2FILE				= Name of the SAS dataset that defines type 2 cohort parameters (washouts, cohortdef, ...)
*	-TYPE3FILE				= Name of the SAS dataset that defines type 3 cohort parameters (washouts, cohortdef, ...)
*	-TYPE4FILE				= Name of the SAS dataset that defines type 4 cohort parameters (washouts, cohortdef, ...)
*   -PREGDUR				= Name of the SAS dataset that contains pregnacy duration delivery codes for type 4 analysis  
*	-TYPE5FILE				= Name of the SAS dataset that defines type 5 cohort parameters (washouts, cohortdef, ...)
*	-TYPE6FILE				= Name of the SAS dataset that defines type 6 cohort parameters (washouts, cohortdef, ...)
*	-METADATAFILE			= Name of the SAS dataset that defines the metadata generated for each time period request for sequential analysis
*   -SURVEILLANCEMODE 		= Indicates if the module should be executed in suveilance mode and which type
*	-COVARIATECODES			= Name of the SAS dataset that includes the codes associated with covariate exctraction
*	-UTILFILE				= Name of the SAS dataset that defines how medical and drug utilization metrics should be calculated
*	-COMORBFILE				= Name of the SAS dataset that defines how CCI/Elixhauser Combined Comorbdity Score should be calculated
*	-DRUGCLASSFILE			= Name of the SAS dataset that lists NDCs by generic and class names and allows 
*					  	  	  the program to count the number of distinct generic names and class names
*	-PROFILE	    		= Indicates if a covariate profile file should be produced
*	-MFUFILE				= Name of the SAS dataset that defines how MFU analysis should be performed
*	-MULTEVENTFILE	 		= Name of the SAS dataset that defines how a multiple events analysis should be performed
*	-MULTEVENTFILE_ADHERE	= Name of the SAS dataset that defines how adherence is calculated for a multiple events analysis
*	-CONCFILE			    = Name of the SAS dataset that defines how concomitant episodes should be created and evaluated
*	-OVERLAPFILE		    = Name of the SAS dataset that defines how secondary episode overlap of primary episodes should be evaluated 
*	-OVERLAPFILE_ADHERE	    = Name of the SAS dataset that defines how adherence is calculated for a overlapping episodes analysis
*	-ITSFILE			    = Name of the SAS dataset that defines parameters for an Interupted Time Series (ITS) analysis
*	-TREATMENTPATHWAYS		= Name of the SAS dataset that defines how the switching module should be performed
*	-MICOHORTFILE			= Name of the SAS dataset that defines how Type 4 PSA/Mother-Infant cohorts will be created
*	-TREEFILE				= Name of the SAS dataset that defines parameters for tree extraction 
*	-TREELOOKUP				= Name of the SAS dataset that defines the parent-child nodes for tree extraction
*	-ICD10ICD9MAP			= Name of the SAS dataset that maps ICD 9 to ICD 10 codes
*	-COVARIATES_CONSIDERED  = For requests calculating a HDPS, specifies the number of variables from each data dimension 
*							  (diagnoses, procedures and drug classes) that are considered for selection as a covariate.
*	-COVARIATES_SELECTED	= For requests calculating a HDPS, specifies the total number of empirically identified covariates 
*							  selected from all the data dimensions combined. 
*	-RANKING				= For requests calculating a HDPS, indicates which model to use for selecting variables
*	-ZERO_CELL_CORR			= For requests calculating a HDPS, indicates whether to screen variables with a zero correction added to each cell in the confounder/outcome 2x2 table.  
*	-HDVARSEL_INPUT			= For requests calculating a HDPS, lists each data dimension dataset                      	
*	-COMPARISON				= Name of the SAS dataset that defines parameters for a PS or MFM analysis
*	-ANALYTICSUBGROUPS		= Name of the SAS dataset that defines subgroups for a PS or MFM analysis
*	-INDLEVEL				= Indicates whether to return an individual level file after PS/MFM matching in the MSOC folder
*	-PERCENTILES			= Indicates the number of percentiles to group propensity scores for percentile analysis
*	-UNCONDITIONAL			= Indicates whether to create risk sets not stratified by matchID. This allows SOC to perform unconditional analyses. 
*	-DIAGNOSTICS			= Indicates if proc logistic diagnostic output files should be generated
*
*
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*	1.4		  01/22/14     DM		Minor bug fix. Added de-duped sort to _agegroups dataset 
*
*	1.5       02/06/15     SL(EM)	Removed MSREQID definition line
*
*	1.6		  02/23/15	   DM		Removed Enr_Start and Enr_end when shaving ITDrugsComb to 
*									avoid a w.a.r.n.i.n.g. in the log and incorrect results in outputs
*									Modified how the required follow-up time after exposure is 
*									calculated
*
*	1.7	 	  03/11_15	   DM		Renamed Episodes_analysis to Pt_Analysis_Episode_Num 
*									Renamed Episodes_exposure to Pt_Exposure_Episode_Num
*									Process _ITDrugs, _ITMeds and _ITLabs with ms_shaveoutside 
*									even when they are empty
*
*	1.8    	  03/26/15	   DM	    Changed the method to verify MSDD file existence before calling 
*				     				the FREEZEDATA macro
*
*	1.9    	  04/13/15	   DM		Changed the stockpiling input file parameters processing
*
*	1.10   	  06/26/15     DM		Verify that patients have at least one day at risk in the look 
*									of their index date (analysis = PS or ADS).
*									Removed Surveillance Analysis related code.
*
*	1.11   	  09/14/15     DM		Added code for Type 4 analysis.
*
*	1.12   	  03/10/16     VC		Enhance Inclusions and Covariates to allow more complex criteria.
*
*	1.13   	  04/21/16	   DM		Carried forward changes made in 2.2.1 version (incidence at day 0)
*
* 	1.14   	  05/26/16	   VC		JIRA 173
*
*	1.15	  06/14/16	   DM		Fix for RxAmt and RxAmt in Type 1 plus hybrid AT/ITT
*
*	1.16	  07/05/16	   DM		Added time to censor data plus fix for JIRA #DEVI-14
*
*	1.17	  08/10/16     AP		Modified to remove true duplicates in SDD (QCI-195)	
*						   DM       Fixed same day claims issue in type 1 analysis (QCI-208)
*
*	1.18	  08/15/16	   DM		Added Surveillance Analysis related code removed in 1.10.
*
*	1.19	  10/17/16	   DM		Fixed an issue regarding Eventcount (QCI-219) and replaced an 
*									append statement with a set one (QCI-210)
*
*	1.20  	  11/14/16	   AP		Added ability to stratify T1/T2 output by geographic location
*	   								Added ability to turn off envelope macro
*	
*	1.21   	  11/30/16	   AP		Added ability to truncate TTE at Query period end date and also to 
*									enumerate reasons for censoring	
*
*	1.22   	  02/09/16	   AP		Fixed bug (QRP-301) in TTE calculation when censor_dth = N	
*
*	2.0	      03/30/17     DM		Reorganization of the code to modularize the POVs creation
*
*	2.1	  	  05/05/17	   AP		1. Extracted hispanic info
*									2. Removed AgeStrat as a run-level parameter and converted to
*										group level parameter
*									3. Moved ms_cidatables, ms_cidatablest3, ms_cidatablest4 into group loop
*	
*   2.2    	  07/12/17     RR       Removed (drop = expireDt) from DoITTCheck
*
*	2.3	 	  06/26/17	   AP		1. Added Type 5 Analysis	
*									2. Integrated MP7 and added new index date definitions
*
*	2.4	      09/14/17	   AP		Bug fix (QRP-437)
*
*   2.5	 	  10/03/17	   AP		-Changed PRECODES to PREGDUR (QRP-388)
*									-Recoded sex values A/U (QRP-453)
*
*	2.6	      12/28/17	   AP		Moved censor table to ms_createcensortable (QRP-462)
*
*	3.0		  05/15/18	   AP		1. Added Type 6 analysis, switching module, multiple events module,
*									   overlap module, and concomitant use module,
*									2. Moved aggregate output outside of Loop and added USERSTRATA parameter
*
*	3.1		  10/20/18	   AP		Added Mom-Baby Linkage Table
*
*	3.2		  11/06/18	   AP		DEV-3254: Added ms_extractmilinkage when combo tool is executed
*
*	3.3		  11/10/18	   AP		Direct temp datasets to dplocal, add timestamps, programmatically determine
*									number of enrollment spans
*
*	3.4		  02/12/19	   AP		Updated requestID and common components parameters to remove MS (i.e. MSReqID --> ReqID)
*
*	4.0		  06/30/19	   AP		Added tree extraction and incorportated PSA/MFM analysis
*
*	4.1		  07/01/19	   AP		Fixed computaton of CumEpisodeLength for Type 6
*
***************************************************************************************************;

%macro ms_cidanum(RUNID=,
	              PROJID=,  
	              WPTYPE=,
	              WPID=,
				  DPID=,
	              VERID=,
	              PERIODIDSTART=,
	              PERIODIDEND=,
	              ANALYSIS=,
	              MONITORINGFILE=,
				  USERSTRATA=,
	              COMBOFILE=,
	              COHORTFILE=,
	              COHORTCODES=,
	              INCLUSIONCODES=,
	              STOCKPILINGFILE=,
				  RUN_ENVELOPE=,
	              FREEZEDATA=,
				  ZIPFILE=,
	              LABSCODEMAP=,
				  DISTINDEX=,
	              TYPE1FILE=,
	              TYPE2FILE=,
	              TYPE3FILE=,
				  TYPE4FILE=,
				  PREGDUR=,	
				  TYPE5FILE=,
				  TYPE6FILE=,
				  METADATAFILE=,
				  SURVEILLANCEMODE=,
	              COVARIATECODES=,
	              UTILFILE=,
	              COMORBFILE=,
	              DRUGCLASSFILE=,
				  PROFILE=,
				  MFUFILE=,
				  MULTEVENTFILE=,
				  MULTEVENTFILE_ADHERE=,
				  CONCFILE=,
				  OVERLAPFILE=,
				  OVERLAPFILE_ADHERE=,
				  ITSFILE=,
				  TREATMENTPATHWAYS=,
				  MICOHORTFILE=,
				  TREEFILE=,
			 	  TREELOOKUP=,
			      ICD10ICD9MAP=,
              	  COVARIATES_CONSIDERED=,
	              COVARIATES_SELECTED=,
	              RANKING=,
	              ZERO_CELL_CORR=,
	              HDVARSEL_INPUT=,                         	
	              COMPARISON= ,
			      ANALYTICSUBGROUPS=,
	              INDLEVEL=,
	              PERCENTILES=,
	              UNCONDITIONAL=,
	              DIAGNOSTICS=);

	%put =====> MACRO CALLED: ms_cidanum v4.1;

	%ms_starttimer(timestart=qrpruntime);

	/*---------------------------------------------------*/
	/*  01 -- Set up variables needed for the run        */
	/*---------------------------------------------------*/
	%LET PROJID=%LOWCASE(&PROJID.);
	%LET WPTYPE=%LOWCASE(&WPTYPE.);
	%LET WPID=%LOWCASE(&WPID.);
	%LET DPID=%LOWCASE(&DPID.);
	%LET VERID=%LOWCASE(&VERID.);
	%LET PERIODIDSTART=%LOWCASE(&PERIODIDSTART.);
	%LET PERIODIDEND=%LOWCASE(&PERIODIDEND.);
	%LET ANALYSIS=%LOWCASE(&ANALYSIS.);
	%LET SURVEILLANCEMODE=%LOWCASE(&SURVEILLANCEMODE.);
	%let PROFILE=%LOWCASE(&PROFILE.);
	%let DISTINDEX=%LOWCASE(&DISTINDEX.);
	%let FREEZEDATA=%LOWCASE(&FREEZEDATA.);

	%LET MONITORINGFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&MONITORINGFILE.), .sas7bdat, %STR())));
	%LET USERSTRATA=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&USERSTRATA.), .sas7bdat, %STR())));
	%LET COHORTFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&COHORTFILE.), .sas7bdat, %STR())));
	%LET TYPE1FILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&TYPE1FILE.), .sas7bdat, %STR())));
	%LET TYPE2FILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&TYPE2FILE.), .sas7bdat, %STR())));
	%LET TYPE3FILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&TYPE3FILE.), .sas7bdat, %STR())));
	%LET TYPE4FILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&TYPE4FILE.), .sas7bdat, %STR())));
	%LET TYPE5FILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&TYPE5FILE.), .sas7bdat, %STR())));
	%LET TYPE6FILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&TYPE6FILE.), .sas7bdat, %STR())));
	%LET METADATAFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&METADATAFILE.), .sas7bdat, %STR())));
	%LET COHORTCODES=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&COHORTCODES.), .sas7bdat, %STR())));
	%LET INCLUSIONCODES=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&INCLUSIONCODES.), .sas7bdat, %STR())));
	%LET COVARIATECODES=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&COVARIATECODES.), .sas7bdat, %STR())));
	%LET MFUFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&MFUFILE.), .sas7bdat, %STR())));
	%LET STOCKPILINGFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&STOCKPILINGFILE.), .sas7bdat, %STR())));
	%LET UTILFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&UTILFILE.), .sas7bdat, %STR())));
	%LET COMORBFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&COMORBFILE.), .sas7bdat, %STR())));
	%LET DRUGCLASSFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&DRUGCLASSFILE.), .sas7bdat, %STR())));
	%LET COMBOFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&COMBOFILE.), .sas7bdat, %STR())));
	%LET PREGDUR=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&PREGDUR.), .sas7bdat, %STR())));
	%LET ZIPFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&ZIPFILE.), .sas7bdat, %STR())));
	%LET MULTEVENTFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&MULTEVENTFILE.), .sas7bdat, %STR())));
	%LET MULTEVENTFILE_ADHERE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&MULTEVENTFILE_ADHERE.), .sas7bdat, %STR())));
	%LET CONCFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&CONCFILE.), .sas7bdat, %STR())));
	%LET OVERLAPFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&OVERLAPFILE.), .sas7bdat, %STR())));
	%LET OVERLAPFILE_ADHERE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&OVERLAPFILE_ADHERE.), .sas7bdat, %STR())));
	%LET ITSFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&ITSFILE.), .sas7bdat, %STR())));
	%LET TREATMENTPATHWAYS=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&TREATMENTPATHWAYS.), .sas7bdat, %STR())));
	%LET MICOHORTFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&MICOHORTFILE.), .sas7bdat, %STR())));
	%LET TREEFILE=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&TREEFILE.), .sas7bdat, %STR())));
	%LET TREELOOKUP=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&TREELOOKUP.), .sas7bdat, %STR())));
	%LET ICD10ICD9MAP=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&ICD10ICD9MAP.), .sas7bdat, %STR())));

	*PS/MFM parameters;
	%let INDLEVEL=%LOWCASE(&INDLEVEL.);
	%let COMPARISON=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&COMPARISON.), .sas7bdat, %STR())));
	%let ANALYTICSUBGROUPS=%TRIM(%SYSFUNC(TranWrd(%LOWCASE(&ANALYTICSUBGROUPS.), .sas7bdat, %STR())));
	%let UNCONDITIONAL=%LOWCASE(&UNCONDITIONAL.);
	%let DIAGNOSTICS=%lowcase(&DIAGNOSTICS.);
	%if %str("&percentiles") = %str("") %then %let percentiles = 10;

	/*---------------------------------------------------*/
	/*  02 -- Import input files in SAS format           */
	/*---------------------------------------------------*/

	%IMPORTFILES(var=&MONITORINGFILE.);
	%IMPORTFILES(var=&USERSTRATA.);
	%IMPORTFILES(var=&COHORTFILE.);
	%IMPORTFILES(var=&TYPE1FILE.);
	%IMPORTFILES(var=&TYPE2FILE.);
	%IMPORTFILES(var=&TYPE3FILE.);
	%IMPORTFILES(var=&TYPE4FILE.);
	%IMPORTFILES(var=&TYPE5FILE.);
	%IMPORTFILES(var=&TYPE6FILE.);
	%IMPORTFILES(var=&METADATAFILE.);
	%IMPORTFILES(var=&COHORTCODES.);
	%IMPORTFILES(var=&INCLUSIONCODES.);
	%IMPORTFILES(var=&COVARIATECODES.);
	%IMPORTFILES(var=&MFUFILE.);
	%IMPORTFILES(var=&STOCKPILINGFILE.);
	%IMPORTFILES(var=&UTILFILE.);
	%IMPORTFILES(var=&COMORBFILE.);
	%IMPORTFILES(var=&DRUGCLASSFILE.);
	%IMPORTFILES(var=&COMBOFILE.);
	%IMPORTFILES(var=&DRUGCLASSFILE.);
	%IMPORTFILES(var=&LABSCODEMAP.);
	%IMPORTFILES(var=&PTSTOEXCLUDE.);
	%IMPORTFILES(var=&PREGDUR.);
	%IMPORTFILES(var=&ZIPFILE.);
	%IMPORTFILES(var=&MULTEVENTFILE.);
	%IMPORTFILES(var=&MULTEVENTFILE_ADHERE.);
	%IMPORTFILES(var=&CONCFILE.);
	%IMPORTFILES(var=&OVERLAPFILE.);
	%IMPORTFILES(var=&OVERLAPFILE_ADHERE.);
	%IMPORTFILES(var=&ITSFILE.);
	%IMPORTFILES(var=&TREATMENTPATHWAYS.);
	%IMPORTFILES(var=&MICOHORTFILE.);
	%IMPORTFILES(var=&TREEFILE.);
	%IMPORTFILES(var=&TREELOOKUP.);
	%IMPORTFILES(var=&ICD10ICD9MAP.);
	%IMPORTFILES(var=&COMPARISON.);
	%IMPORTFILES(var=&ANALYTICSUBGROUPS.);

	/*---------------------------------------------------*/
	/*  03 -- Initialize macro variables and 			 */
	/*        Process input file parameters              */
	/*---------------------------------------------------*/

	*Initialize macro variables;
	%ms_initmacrovariables();

	*Process input files;
	%ms_processinputfiles();

	/*---------------------------------------------------*/
	/*  04 -- Process Ranges "-" and Wilcards "*"        */
	/*        in lookup codes                            */
	/*---------------------------------------------------*/

	%ISDATA(dataset=_diag);
	%IF %EVAL(&NOBS.>=1) %THEN %DO;
	    %ms_processwildcards(InFile=_diag, 
	                         CodeVar=code, 
	                         OutFile=_diag);
	%END;

	/*---------------------------------------------------*/
	/*  05 -- Extract Raw Data and run envelope or       */
	/*        Create combo records (if applicable)       */
	/*---------------------------------------------------*/
	%if %str(&COMBOFILE.) = %str() or %length(&COMBOFILE.) = 0. %then %do;	

	   *Programming note:  We are not using caresettingprincipal in the pre-extraction phase because
	    typical sensitivity analysis around caresettingprincipal could lead to massive claim duplication;

	   %ms_extractmeds(datafile=indata.&proctable.,
	                   datacode=PX ,
	                   datacodetype=PX_codetype ,
	                   lookfile=_proc, 
	                   outfile=dplocal._procextract(drop=EncounterID Provider));

		proc sort data=dplocal._procextract nodupkey;
		by _all_;
		run;

	   *Delete data from patients in DPs &PTSTOEXCLUDE. files; 
	   %ms_delpatients(datafile=dplocal._procextract,
	                   ptsfile=&PTSTOEXCLUDE.,
	                   Outfile=dplocal._procextract);

	   %ms_extractmeds(datafile=indata.&diatable.,
	                   datacode=DX,
	                   datacodetype=DX_codetype,
	                   lookfile=_diag, 
	                   outfile=dplocal._diagextract(drop=EncounterID Provider));

		proc sort data=dplocal._diagextract nodupkey;
		by _all_;
		run;

	   *Delete data from patients in DPs &PTSTOEXCLUDE. files; 
	   %ms_delpatients(datafile=dplocal._diagextract,
	                   ptsfile=&PTSTOEXCLUDE.,
	                   Outfile=dplocal._diagextract);

	    *Codes must be unique to avoid claim duplication.;
	    proc sort nodupkey data=_ndc;
	    by code;
	    run;

	    %ms_extractdrugs(datafile=indata.&distable.,
	                     datavar=NDC ,
	                     lookfile=_ndc(drop=codecat CodeType), 
	                     lookvar=code , 
	                     outfile=drugs);

		proc sort data=drugs out=dplocal.drugs nodupkey;
		by _all_;
		run;

		proc datasets nowarn noprint lib=work;
			delete drugs;
		quit;

	   *Delete data from patients in DPs &PTSTOEXCLUDE. files; 
	   %ms_delpatients(datafile=dplocal.drugs,
	                   ptsfile=&PTSTOEXCLUDE.,
	                   Outfile=dplocal.drugs);
	                   

	   %ms_extractlabs(datafile=indata.&labtable.,
	                   lookfile=_lab, 
	                   outfile=dplocal.labextract);

	   proc sort data=dplocal.labextract nodupkey;
	   by _all_;
	   run;

	   *Delete data for patients in DPs &PTSTOEXCLUDE. files; 
	   %ms_delpatients(datafile=dplocal.labextract,
	                   ptsfile=&PTSTOEXCLUDE.,
	                   Outfile=dplocal.labextract);   

		%ms_extractmilinkage(datafile=indata.&miltable.,
							 lookfile=_mil,
							 outfile=dplocal.milextract);

	   *Delete data for patients in DPs &PTSTOEXCLUDE. files; 
	   %ms_delpatients(datafile=dplocal.milextract,
	                   ptsfile=&PTSTOEXCLUDE.,
	                   Outfile=dplocal.milextract);   	
	%end;
	%else %do;
	   *Combo is called using a macro wrapper to preserve the scoping of 
	    macro call combo variables (specifically STOCKPILINGFILE);
	    %MACRO RUNCOMBO;        
	        %let COMBORUN=1;
			 %let DXENVEL = 1;
	         %let PXENVEL = 1;
	        %if %EVAL(&RUN_ENVELOPE. = 2) %then %do;
	            %let DXENVEL = 0;
	            %let PXENVEL = 0;
	        %end;
	        %combo(COMBFILE=&COMBOFILE.,
	               CODESFILE=&COMBOFILE.Codes,
	               DXENVEL=&DXENVEL.,
	               PXENVEL=&PXENVEL.,
	               OUTNAME=&RUNID._Combo,
	               STOCKPILING=1,
	               STOCKPILINGFILE=&COMBOFILE.Stock,
	               ENROLGAP=0,
	               LABSCODEMAP=&LABSCODEMAP.,
	               SAVETODPLOCAL=N,
	               PTSTOEXCLUDE=&PTSTOEXCLUDE.);

	        *Combo creates _Drugs, but we need Drugs to continue;
	        proc datasets library=dplocal nowarn nolist;
	        change _drugs=drugs;
	        quit;
	    %MEND RUNCOMBO;
	    %RUNCOMBO;

		*MIL table not processed by combo tool;
		%ms_extractmilinkage(datafile=indata.&miltable.,
							 lookfile=_mil,
							 outfile=dplocal.milextract);

	   *Delete data for patients in DPs &PTSTOEXCLUDE. files; 
	   %ms_delpatients(datafile=dplocal.milextract,
	                   ptsfile=&PTSTOEXCLUDE.,
	                   Outfile=dplocal.milextract);   	

	%end;

	* Deaths are not processed by combo tool. Therefore they must be extracted no matter if combo is run;
	%ms_extractdeaths(deathfile=indata.&DEATHTABLE.,
					  encounterfile=indata.&enctable.,
					  outfile=dplocal.deathextract);

	* Delete data for patients in DPs &PTSTOEXCLUDE. files; 
	%ms_delpatients(datafile=dplocal.deathextract,
	                ptsfile=&PTSTOEXCLUDE.,
	                Outfile=dplocal.deathextract);	

	*Combine _diags and _procs for envelope later;
	proc sql noprint;
	create table dplocal.meds as 
	select proc.PatId,
	       proc.Adate,
	       compress(proc.PX,' .')as Code length=11,
	       "  " as Pdx,
	       proc.EncType,
	       PX_Codetype as Codetype,
	       "PX" as CodeCat,
		   1 as RxSup,
	       1 as RxAmt
	from dplocal._procextract as proc
	union corr all
	select diag.PatId,
	       diag.Adate,
	       compress(diag.DX,' .') as Code length=11,
	       diag.EncType,
	       diag.Pdx,
	       diag.DX_Codetype as Codetype,
	       "DX" as CodeCat,
		   1 as RxSup,
	       1 as RxAmt
	from dplocal._diagextract as diag;
	quit; 

	*To avoid w.a.r.n.i.n.g.s. down the line;
	data dplocal.meds;
	set dplocal.meds;
	length Adate 4.;
	run;

	*If meds records not out of combo, run envellope;
	%ISDATA(dataset=dplocal.meds);
	%if (%str(&COMBOFILE.) = %str() or %length(&COMBOFILE.) = 0.) & %EVAL(&NOBS.>=1) & %EVAL(&RUN_ENVELOPE. ne 2) %then %do;	
	    %MS_ENVELOPE(INFILE=dplocal.meds,		                /*Name of SAS dataset with diagnosis and/or procedure claims data*/
	    	         ENCOUNTERINFILE=indata.&ENCTABLE.,			/*Name of SAS dataset with encounter data*/
	    	         OUTFILE=dplocal.meds);      	            /*Name of output file with reclassified claims*/ 
	%end;

	*Separate raw un-stockpiled RX claims from 
	 combo claims set to behave like Rx;
	data dplocal.drugs
	     dplocal.drugsComb;
	set dplocal.drugs;
	if RxSup>0 and RxAMt>0;
	if comb then output dplocal.drugsComb;
	else output dplocal.drugs;
	run;

	proc datasets library=work nowarn nolist;
	delete _:;
	quit;

	/*---------------------------------------------------*/
	/*  06 -- Reconcile all enrollment spans		     */ 
	/*---------------------------------------------------*/

	%ms_create_enrollment_spans();

	/*---------------------------------------------------*/
	/*  07 -- Loop through Group(s)			             */ 
	/*---------------------------------------------------*/
	
		/*Output preproccessing runtime*/
		%ms_stoptimer(timestart=qrpruntime);
		%ms_outputruntimes(timevar=qrpruntime, step=%str(Preprocessing/code extraction), group=, monitoringperiod=);

	%macro forloop;

	%do Group=1 %to &Ngroup.;

	/*---------------------------------------------------*/
	/*  07.a -- Loop pre-processing                      */ 
	/*---------------------------------------------------*/

		/*loop timing*/
		%ms_starttimer(timestart=loopruntime);
		
	    *Set current Group macro parameters;
		%ms_setnumloopmacrovars();

		%PUT INFO: NOW LOOPING ON &ITGROUP.;
		%PUT &ITGROUP. processing began on %sysfunc(date(),date9.) %sysfunc(time(),tod8.);

		*Set query period for data driven queries;
		%if "&datadrivenqueryperiod" = "Y" %then %do;
			%ms_setdatadrivenqueryperiod();
		%end;

	    *Get specific codes for this loop and separate diags from others to process wildcards;
	    data _diag 
	         _others;
		    set &COHORTCODES.;
		    where group="&ITGROUP.";
		    if CodeCat in("DX") then output _diag ;
		    else output _others;
	    run;

	    %ISDATA(dataset=_diag);
	    %IF %EVAL(&NOBS.>=1) %THEN %DO;
	        %MS_ProcessWildcards(InFile=_diag, 
	                             CodeVar=code, 
	                             OutFile=_diag);
	    %END;

	    *Create Loop-specific parameter files;
	    data _ITGroupParamNDC
	         _ITGroupParamMeds
	         _ITGroupParamLabs(rename=Group=RawGroup rename=CareSettingprincipal=RawCaresetting)
			 _ITGroupParamDate
			 _ITGroupParamMIL;
			set _diag
		    	_others;

			*Defensive coding;
			if Condinclusion=. then Condinclusion=1;
			if SubCondinclusion=. then SubCondinclusion=1;
			if Cond=. then Cond=-1;
			if SubCond=. then SubCond=-1;	
			if codedays = . then codedays = 1;
			if missing(excludesupply) then excludesupply = 'N';
			excludesupply=upcase(excludesupply);

			*Output;
			if CodeCat="RX"          then output _ITGroupParamNDC;
			if CodeCat in("DX","PX") then output _ITGroupParamMeds;
			if CodeCat in("LB")      then output _ITGroupParamLabs;
			if CodeCat in("AN","DT") then output _ITGroupParamDate;
			if CodeCat in("MI")      then output _ITGroupParamMIL;
	    run;

		*Indicator if group index determined by Age Anniversary or Calendar Date;
		%let agedatecohort = N;
		%isdata(dataset=_ITGroupParamDate);
		%if %eval(&nobs.) > 0 %then %do;
			%let agedatecohort = Y;
		%end;

	    *Break out CaresettingPrincipal into Caresetting and Principal;
	    %ms_caresettingprincipal(InFile=_ITGroupParamMeds, 
	                             Var=CareSettingPrincipal, 
	                             OutFile=_ITGroupParamMeds);

	/*---------------------------------------------------*/
	/*  07.b -- Extract loop-specific records            */ 
	/*---------------------------------------------------*/

	    *Extract Records necessary for creating this group only
	     (record duplic OK + add loop parameters back to records); 
	    %ms_loopmeds(datafile=dplocal.meds,
	                 lookfile=_ITGroupParamMeds, 
					 multiplecodecat=Y,
	                 outfile=_ITMeds);

	    %ms_extractdrugs(datafile=dplocal.drugs,
	                     datavar=NDC ,
	                     lookfile=_ITGroupParamNDC, 
	                     lookvar=code , 
	                     outfile=_ITDrugs);
		
		*Need to define Supply End Date to make sure days of supply are constrained within enrollment;
		data _ITDrugs;
		set _ITDrugs;
		format RxSupEndDt date9.;
		if not missing(codesupply) then RxSup = CodeSupply;
		RxSupEndDt=Rxdate+RxSup-1;
		run;

	    %ms_extractdrugs(datafile=dplocal.drugscomb,
	                     datavar=NDC ,
	                     lookfile=_ITGroupParamNDC, 
	                     lookvar=code , 
	                     outfile=_ITDrugsComb);

		*Need to define Supply End Date to make sure days of supply are constrained within enrollment;
		data _ITDrugsComb;
		set _ITDrugsComb;
		format RxSupEndDt date9.;
		if not missing(codesupply) then RxSup = CodeSupply;
		RxSupEndDt=Rxdate+RxSup-1;
		run;

	    %ms_extractlabs(datafile=dplocal.labextract,
	                    lookfile=_ITGroupParamLabs, 
	                    outfile=_ITlabs);	

	 	%ms_extractmilinkage(datafile=dplocal.milextract,
							 lookfile=_ITGroupParamMIL,
							 outfile=_ITMIL);

	/*---------------------------------------*/
	/*  07.c -- Shave enrollment spans		 */
	/*---------------------------------------*/    

	    *Make sure all claims overlap enrollment span and only 
	     keep eligible exposures (overlaping Enr_Start Enr_End);
	  	%ms_shaveoutside(reffile=dplocal.enr_&enrollmentnum.,
	                     refstart=Enr_Start,
	                     refend=Enr_End,
	                     KeepPartBf=N,
	                     ToShaveFile=_ITDrugs,
	                     ToShaveStart=RxDate,
	                     ToShaveEnd=RxSupEndDt,
	                     outfile=_ITDrugs);
	   
		%ISDATA(dataset=_ITDrugsComb);
		%IF %EVAL(&NOBS.>=1) %THEN %DO;
	    %ms_shaveoutside(reffile=dplocal.enr_&enrollmentnum.,
	                     refstart=Enr_Start,
	                     refend=Enr_End,
	                     KeepPartBf=N,
	                     ToShaveFile=_ITDrugsComb,
	                     ToShaveStart=RxDate,
	                     ToShaveEnd=RxSupEndDt,
	                     outfile=_ITDrugsComb(drop=Enr_Start Enr_end));
	    %END;
	    
	    %ms_shaveoutside(reffile=dplocal.enr_&enrollmentnum.,
	                     refstart=Enr_Start,
	                     refend=Enr_End,
	                     KeepPartBf=N,
	                     ToShaveFile=_ITMeds,
	                     ToShaveStart=ADate,
	                     ToShaveEnd=ADate,
	                     outfile=_ITMeds);
	    
	  	%ms_shaveoutside(reffile=dplocal.enr_&enrollmentnum.,
	                   refstart=Enr_Start,
	                   refend=Enr_End,
	                   KeepPartBf=N,
	                   ToShaveFile=_ITLabs,
	                   ToShaveStart=ADate,
	                   ToShaveEnd=ADate,
	                   outfile=_ITLabs);   

		*For MIL analysis, Mother must be enrolled on ADate. Infant claim not shaved;
	  	%ms_shaveoutside(reffile=dplocal.enr_&enrollmentnum.,
	                   refstart=Enr_Start,
	                   refend=Enr_End,
	                   KeepPartBf=N,
	                   ToShaveFile=_ITMil,
	                   ToShaveStart=ADate,
	                   ToShaveEnd=ADate,
	                   outfile=_ITMIL);  
	    
		/*For Type 6 analysis, need to select earliest date for same-day dispensings before getting resolved in %ms_stockpiling.
		 if uptakedate = COMPUTEDSTARTMARKETINGDATE, can ignore, because specific COMPUTEDSTARTMARKETINGDATE is GROUP and not NDC specific*/
		%if %eval(&type.=6) & %upcase("&UPTAKEDATE.") ne "COMPUTEDSTARTMARKETINGDATE" %then %do;
			%isdata(dataset=_itdrugs);   
			%if %eval(&nobs.>0) %then %do;
			proc means data=_itdrugs noprint nway;
				var &uptakedate.;
				class patid rxdate t6_index;
				output out=_mindates(drop=_:) min=;
			run;

			proc sql noprint undo_policy=none;
				create table _itdrugs as
				select x.*,
					   y.&uptakedate.
				from _itdrugs(drop=&uptakedate.) as x,
					 _mindates as y
				where x.patid = y.patid and x.rxdate = y.rxdate and x.t6_index=y.t6_index;
			quit;
			%end;
		%end;

		/*Type 3*/
		%if %str(&ExclOnSameDay.)=%str(Y) %then %do;
	        *Identify SameDayDispensings for different NDCs but from the same group;
	        proc sort nodupkey data=_ITDrugs out=_SameDayNDCs(keep=Patid rxdate ndc);
	        by Patid rxdate ndc;
			WHERE T&type._INDEX="DEF";
	        run;

	        proc means data=_SameDayNDCs nway noprint;
	        class Patid rxdate;
	        output out=_SameDayNDCs(where=(_freq_>1));
	        run;		
	    %end;  

	/*--------------------------------------------------------------*/
	/*  07.d -- Create claims dataset based on age or calendar date */ 
	/*--------------------------------------------------------------*/

		%ms_extractanniversarydate(datafile = dplocal.enr_&enrollmentnum.,
									lookfile = _ITGroupParamDate,
									lookvar = code,
									outfile = _itdates);

	/*---------------------------------------------------*/
	/*  07.e -- Apply StockPiling                        */ 
	/*---------------------------------------------------*/
	    
	    %ISDATA(dataset=infolder.&STOCKPILINGFILE.);	
	    %IF %EVAL(&NOBS.>0) %THEN %DO;
	        data &STOCKPILINGFILE.;
	        set infolder.&STOCKPILINGFILE.;
	        where group="&ITGROUP.";
	        call symputx("SAMEDAY",strip(SameDay));
	        call symputx("SUPRANGE",strip(SupRange));
	        call symputx("AMTRANGE",strip(AmtRange));
	        call symputx("PERCENTDAYS",put(PercentDays,best.));
	        run;
	    %end;         

	    %MS_STOCKPILING(INFILE=_ITDrugs,	  		
	                    CLMDATE=RxDate,                            		
	                    CLMSUP=RxSup,                              		
	                    CLMAMT=RxAmt,	                           		
	        		    PROCFLAG=,                                 		
	        		    PERCENTDAYS=&PERCENTDAYS.,                          		
	        		    GROUPING=Group StockGroup T1_INDEX T2_INDEX T2_FUP T3_INDEX T3_FUP T4_INDEX T4_FUP T5_INDEX T6_INDEX CONC_FUP codedays excludesupply &InclExclVars.,                   		
	        		    SAMEDAY=&SAMEDAY.,                                 	
	        			SUPRANGE=&SUPRANGE.,									
	        			AMTRANGE=&AMTRANGE.,
	        		    ID=CODECAT &t6vars.,                                       		
	        		    OUTFILE=_ITDrugs,  		
	        			OUTFILEEXCL=_ITDrugsExcl
	                    );

	    data _ITDrugs;
	    set  _ITDrugs
	         _ITDrugsComb(in=a);
	    *ExpireDt does not exist in _ITDrugsComb;
	    if a then ExpireDt=Rxdate+RxSup-1; 	
	    run;

		*Need to reshave because Stockpiling may have pushed claims outside enrollment period;
	    %ISDATA(dataset=_ITDrugs);
		%IF %EVAL(&NOBS.>=1) %THEN %DO;
	        %ms_shaveoutside(reffile=dplocal.enr_&enrollmentnum.,
	                       refstart=Enr_Start,
	                       refend=Enr_End,
	                       KeepPartBf=N,
	                       ToShaveFile=_ITDrugs,
	                       ToShaveStart=RxDate,
	                       ToShaveEnd=ExpireDt,
	                       outfile=_ITDrugs);
	    %END;	

	    proc datasets library=work nowarn nolist;
	    delete _diag _others;
	    quit;

	/*---------------------------------------------------*/
	/*  07.f -- Create raw POVs datasets                 */
	/*---------------------------------------------------*/
		data _GroupIndex(keep=PatId group Adate ExpireDt RXsup RXAmt NumDispensing Enr_Start Enr_end productapprovaldate /* POV1: Primary cohort Group Index Date	(GroupIndex) */
							  productmarketingdate otherproductdate codecat codetype code stockgroup enctype pdx linkage 
							  csex linkedid cbirth_date cenr_start birth_type matchmethod) 		
			_GroupWash(keep=PatId Adate ExpireDt Enr_Start Excludesupply)              									 /* POV2: Primary Cohort Group Washout  	(GroupWash)   */								
			_GroupWashForTrunk(keep=PatId Adate Enr_Start)																 /* POV4: Primary Cohort Truncation  		(GroupTrunk)  */
	        _InclExcl                                                             										 /* POV3: Inclusion/Exclusion 				(InclExcl)    */
			_FUPEvent(keep=PatId Adate ExpireDt Enr_Start codecat codetype code Excludesupply stockgroup enctype pdx)    /* POV5: Risk Follow-Up Event   			(FUPEvent)    */
	        _FUPWash(keep=PatId Adate ExpireDt Enr_Start Excludesupply)		                 							 /* POV6: Risk Follow-Up Washout 			(FUPWash)     */

			/*Type 4 files*/
			%if %eval(&type.=4) %then %do;
			%do n=1 %to &NUM_MPN.;
			_GroupIndex&n.(keep=PatId Adate ExpireDt NumDispensing RXsup RXAmt Enr_Start Enr_end CodeSupply codecat 
							codetype code stockgroup enctype pdx) 														/* POV1: Primary cohort Group Index Date	(GroupIndex) */
			_GroupWash&n.(keep=PatId Adate ExpireDt Enr_Start Excludesupply)											/* POV2: Primary Cohort Group Washout 		(GroupWash)  */
			%end;
			_GroupPreg(keep=PatId Adate ExpireDt Enr_Start CodeCat Codetype Code PriorityGroup1 PriorityGroup2 			/* Pregnancy cohort Group duration def	  	(GroupPreg)	 */
			 				Priority Duration rename=code=pregdurcode)           
			%end;

			/*Type 6 files*/
			%if %eval(&type.=6) %then %do;
			_GroupIndex2(keep=PatId Adate ExpireDt RXsup RXAmt NumDispensing Enr_Start Enr_end 							/* POV1: Secondary Cohort Group Index Date	(GroupIndex2) */
							  productapprovaldate productmarketingdate otherproductdate code)
			%end;
			;

			length code $14;

		    set _ITDrugs(in=a rename=RxDate=ADate rename=ndc=code)
		        _ITMeds(in=b)
		        _ITLabs(in=c rename=rawgroup=group)
				_itdates(in=d)
				_itmil(in=e);

			if b or c or d or e then do;
				if not missing(codesupply) then RxSup = CodeSupply;
				else RxSup = 1;
			    Expiredt=Adate+RxSup-1;
			    RXAmt=1;
				NumDispensing = 1;
		    end;

			if e then do;
				linkage = 1;
			end;
			else do;
				linkage = 0;
			end;

			%if %eval(&type.=1) %then %do;
	            /*POV1*/ if T1_INDEX in('DEF')       then output _GroupIndex;
	            /*POV2*/ if T1_INDEX in('IOC')       then output _GroupWash;
	            /*POV3*/ if T1_INDEX in('INC','EXC') then output _InclExcl;            
	        %end;

			%if %eval(&type.=2) %then %do;
	            /*POV1*/ if T2_INDEX in('DEF')       then output _GroupIndex;
	            /*POV2*/ if T2_INDEX in('IOT','IOD') then output _GroupWash;  		 *for the IO* part;
			    /*POV4*/ if T2_INDEX in('IOT','FUT') then output _GroupWashForTrunk; *for the **T part;
			    /*POV3*/ if T2_INDEX in('INC','EXC') then output _InclExcl;
	            /*POV5*/ if T2_FUP in('DEF') 		 then output _FUPEvent;
	            /*POV6*/ if T2_FUP in('IOC') 		 then output _FUPWash;
	        %end;

			%if %eval(&type.=3) %then %do;
	            /*POV1*/ if T3_INDEX in('DEF')		 then output _GroupIndex;
	            /*POV2*/ if T3_INDEX in('IOC') 		 then output _GroupWash; 		 *for the IO* part;
	            /*POV3*/ if T3_INDEX in('INC','EXC') then output _InclExcl;
	            /*POV5*/ if T3_FUP in('DEF') 		 then output _FUPEvent;
	            /*POV6*/ if T3_FUP in('IOC') 		 then output _FUPWash;
	        %end;

			%if %eval(&type.=4) %then %do;
		        /*POV1*/ if T4_INDEX in('DEF')       then output _GroupIndex;
					%do n=1 %to &NUM_MPN.;
				/*POV1*/ if T4_INDEX in("MP&n.")     then output _GroupIndex&n.;
					%end;
		        /*POV2*/ if T4_INDEX in('IOC') 		 then output _GroupWash; 		 *for the IO* part;
					%do n=1 %to &NUM_MPN.;
				/*POV2*/ if T4_INDEX in("IC&n.")     then output _GroupWash&n.;
					%end;
		        /*POV3*/ if T4_INDEX in('INC','EXC') then output _InclExcl;
				/*Preg*/ if T4_INDEX in('PCY')       then output _GroupPreg;
		        /*POV5*/ if T4_FUP in('DEF') 		 then output _FUPEvent;
		        /*POV6*/ if T4_FUP in('IOC') 		 then output _FUPWash;								
	        %end;

			%if %eval(&type.=5) %then %do;
	            /*POV1*/ if T5_INDEX in('DEF')       then output _GroupIndex;
	            /*POV2*/ if T5_INDEX in('IOT','IOD') then output _GroupWash;         *for the IO* part;
	            /*POV4*/ if T5_INDEX in('IOT','FUT') then output _GroupWashForTrunk; *for the **T part;
	            /*POV3*/ if T5_INDEX in('INC','EXC') then output _InclExcl;
	        %end;

			%if %eval(&type. eq 6) %then %do;
	            /*POV1*/ if T6_INDEX in('DEF') 		 then output _GroupIndex _GroupIndex2;
	            /*POV2*/ if T6_INDEX in('IOD') 		 then output _GroupWash;         *for the IO* part;
	            /*POV3*/ if T6_INDEX in('INC','EXC') then output _InclExcl;
	        %end;
		run;

		proc datasets library= work nowarn nolist;
	   		delete _ITDrugs _ITMeds _ITLabs _ITDates;
	    quit;

	/*---------------------------------------------------*/
	/*  07.g -- POV1: Find All potential index dates     */
	/*---------------------------------------------------*/

		*Leave CohortId blank for primary cohort;
		%ms_createpov1(CohortId=);

		%IF %EVAL(&type.=4) %THEN %DO;
			%ms_createpov1t4(CohortId=);
			%do n=1 %to &NUM_MPN.;
				%ms_createpov1(CohortId=&n.);
			%end;
		%END;

	/*---------------------------------------------------*/
	/*  07.h -- POV2: identify all index dates that are  */
	/*          free of POV2 records in washper          */
	/*---------------------------------------------------*/

		*Leave CohortId blank for primary cohort;
		%ms_createpov2(CohortId=);

		%IF %EVAL(&type.=4) %THEN %DO;
			%do n=1 %to &NUM_MPN.;
				%ms_createpov2(CohortId=&n.);
			%end;
		%END;

	/*---------------------------------------------------*/
	/*  07.i -- POV3: identify all index dates meeting   */
	/*          inclusion Exclusion                      */
	/*---------------------------------------------------*/

		*POV3 will be evaluated for prevalent Type 6 episodes in %ms_createt6episodes;
		%if %eval(&type. ne 6) %then %do;
		%ms_createpov3(CohortId=);    
		%end;

	/*---------------------------------------------------*/
	/*  07.j -- POV4: Create At-Risk Episodes            */
	/*---------------------------------------------------*/

		*Before we build episodes, we must make sure potential index dates are  
		 separated by at least ITT days if ITT episodes are required;
		%macro DoITTCheck(CohortId=);
			%let paramsuffix=;
			%if %str("&CohortId.") ne %str("") %then %do;
				%let paramsuffix=2;
			%end;

			%if %eval(&&itt&paramsuffix. ne 0) %then %do;
				*When ITT days is greater than exposure plus washper we must make sure that two consecutive  
		         claims are separated by at least ITT days for the second one to be a valid index date;
				data _POV1&CohortId.;
		        set _POV1&CohortId.;
		        ExpireDt=ADate;	
		        run;

				%ms_findgap(InFile=_POV1&CohortId.,
		                    OutFile=_POV1&CohortId., 
		                    sort=PatId Adate descending ExpireDt,
		                    NoDup=Y, 
		                    gapby=PatId, 
		                    gap=ittdays&paramsuffix.-1,
							dupby=patid adate);

				*Reset ExpireDt to first claim ADate plus supply;
				data _POV1&CohortId.;
		        set _POV1&CohortId.;
		        ExpireDt=ADate+RxSup-1;
		        run;

				%if %str("&CohortId.") ne %str("") %then %do;
				proc sort data=_GroupIndex&CohortId.;
					by patid adate;
				run;
				%end;

				*Clean potential index date dataset for later use;
				data _GroupIndex&CohortId.;
		        merge _POV1&CohortId. (keep=patid adate in=a)
		              _GroupIndex&CohortId.;
		        by PatId Adate;
		        if a;	    	
		        run;
			%end;
		%mend DoITTCheck;

		%DoITTCheck(CohortId=);

		*Leave CohortId blank for primary cohort;
		%IF %EVAL(&type. ne 4) %THEN %DO;		
			%ms_createpov4(CohortId=);
		%END;
		%ELSE %DO;		
			%ms_createpov4t4(CohortId=);

			%do n=1 %to &NUM_MPN.;
				%DoITTCheck(CohortId=&n.);
				%ms_createpov4(CohortId=&n.);
			%end;						
		%END;	

	/******************************************/
	/* For Type 6 - select prevalent episodes */
	/******************************************/
		%if %eval(&type=6) %then %do;
			/*POV1*/
			%ms_createpov1(CohortId=2);

			proc datasets nowarn noprint lib=work;
				delete _agegroups;
				change _AgeGroups2 = _AgeGroups;
			quit;
			/*POV3*/
			%ms_createpov3(cohortID=2);    
			/*POV4*/
			%ms_createpov4(CohortId=2);
		%end;

	/*---------------------------------------------------*/
	/*  07.k -- Create Final Record wrt index date       */
	/*---------------------------------------------------*/
		%ms_createptsmasterlist();
		
	/**********/
	/* EVENTS */
	/**********/

		%ms_createpov56();

	/********************************************************************/
	/* For Type 5 - select index episode and create subsequent episodes */
	/********************************************************************/
		%if %eval(&type=5) %then %do;
			%ms_createt5episodes();
		%end;

	/*---------------------------------------------------*/
	/*  07.l -- Finalize Episode List			         */
	/*---------------------------------------------------*/
	    
		%ms_finalizeptsmasterlist();

		%if %eval(&type=4) %then %do;
			* Create the control group (match treated patients);
			%ms_createnopreggroup();

			* Retain MOI episodes/codes during pregnancy/control episode;
			%ms_finalizemoiepisodes();
		%end;


	/*---------------------------------------------------*/
	/*  07.m -- Calculate additional metrics	         */
	/*---------------------------------------------------*/

	 	%if %eval(&type < 3) %then %do; 
			*Compute Denominators;
			%ms_cidadenom();
		%end;

		%if %eval(&type = 2) %then %do; 
			*Compute never exposed Cohort;
			%ms_nvrexposed();
		%end;
		%if %eval(&type = 4) %then %do; 
			*Compute MOI stats;
			%ms_moimetrics(cohortfile=_PtsMasterList, outfile=mstr_preg);
			%if "&t4_nopreggroup." = "Y" %then %do;
			%ms_moimetrics(cohortfile=_ControlGroup, outfile=mstr_nopreg);
			%end;
		%end;

		*Run surveillance if required and applicable;
		%if ("&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p") and %eval(&type. = 2) %then %do;

			%if ("&SURVEILLANCEMODE."="f") %then %do;
				data _PtsMasterList;
				set _PtsMasterList;
				format EndFollowUpType $20.;
				if LastLookFollowed=0 then do; *apply only to eligible patients;
				  if min(Feventdt,EpisodeEndDt) < &enddate. then do;
				      LastLookFollowedDt = &enddate.; 
				      LastLookFollowed = &PERIODIDSTART.;
				      EndFollowUpType="EpisodeEndBFEnddate";
				  end;
				  else if Feventdt = &enddate. then do;
				      LastLookFollowedDt = &enddate.; 
				      LastLookFollowed = &PERIODIDSTART.;
				      EndFollowUpType="EventOnEnddate";
				  end;
				end;
				run;
			%end;

			* Prior data existence for Surveillance;
			%ISDATA(dataset=DPLPRIOR.&RUNID._mstr);	
			%IF %EVAL(&NOBS.>0) and %EVAL(&PERIODIDSTART. > 1) %THEN %DO;
				%ms_surveillance();
			%END; 
		%end;

		*Codes that define exposure/analytic cohorts;
		%if "&DISTINDEX." = "y" %then %do;
			%ms_codedistribution();
		%end;

		*Type 6;
		%if %eval(&type=6) %then %do;
			/*Calculate dates for [runid]_T6_ProductDates*/
				/*1. ProductApprovalDate, ProductMarketingDate, Otherproductdate*/
				data _ITGROUPPARAM_Combined;
					set _itgroupparammeds(keep=productmarketingdate productapprovaldate Otherproductdate group)
						_itgroupparamndc(keep=productmarketingdate productapprovaldate Otherproductdate group)
						_itgroupparamlabs(keep=productmarketingdate productapprovaldate Otherproductdate rawgroup rename=rawgroup=group);
				run;
				proc sql noprint;
					create table _productdates1 as
					select  min(productmarketingdate) as productmarketingdate format=date9.
						  , min(productapprovaldate) as productapprovaldate format=date9.
						  , min(Otherproductdate) as Otherproductdate format=date9.,
						  "&ITGROUP." as group format=$40.
				from _ITGROUPPARAM_Combined;
				quit;

				/*2. ComputedStartMarketingDate*/
				%isdata(dataset=_PtsMasterList);
				%if %eval(&NOBS.>=1) %then %do;
				proc sql noprint;
					create table _productdates2 as
					select min(indexdt) as computedstartmarketingdate format=date9.,
							"&ITGROUP." as group format=$40.
					from _PtsMasterList;
				quit;
				%end;
				%else %do;
					data _productdates2;
						format group $40. computedstartmarketingdate date9.;
						computedstartmarketingdate = .;
						group = "&ITGROUP.";
					run;
				%end;

				data _productdates_group;
					format group $40.;
					merge _productdates1 _productdates2;
					by group;
				run;

			/*Store to MSOC*/
			%IF %EVAL(&group.=1) %THEN %DO;
	   	 		data MSOC.&RUNID._T6_productsdates;
	        		set _productdates_group;
	        	run;
			%end; 
			%else %do;
	   	 		data MSOC.&RUNID._T6_productsdates;
					set MSOC.&RUNID._T6_productsdates
	 					_productdates_group;
				run;
			%end; 

			proc sort data=_PtsMasterList; by group; run;

			/*Add computedstartmarketingdate to _PtsMasterList*/
			data _PtsMasterList;
				merge _PtsMasterList (in=a)
					_productdates_group (keep=group computedstartmarketingdate);
				by group;
				if a;
			run;

			/*Calculate total time on treatment*/
			proc means data=_PtsMasterList noprint nway;
				var tte;
				class patid;
				output out=_totaltte(drop=_:) sum=CumEpisodeLength;
				where incexl = 1; 
			run;
				
			proc sort data=_PtsMasterList; 
				by patid;
			run;

			/*Update [runid]_mstr with total time on treatment*/
			data _PtsMasterList;
				merge _PtsMasterList _totaltte;
				by patid;
			run;
		%end;

		/***************************************************/
		/** ALL Types - Store patient episodes to DPLocal **/
		/***************************************************/
	    %IF %EVAL(&group.=1) %THEN %DO;

			%if %eval(&type ne 4) %then %do;
		        data DPLocal.&RUNID._mstr;
		        set _PtsMasterList;
		        run;
			%end;

			%if %eval(&type eq 4) %then %do; 
				data DPLocal.&RUNID._mstr_preg;
		        set _PtsMasterList;
		        run;

				data DPLocal.&RUNID._sec;
		        set _PtsMasterList2;
		        run;
				%if "&t4_nopreggroup." = "Y" %then %do;
					data DPLocal.&RUNID._mstr_nopreg;
			        set _ControlGroup;
			        run;
				%end;
				data DPLocal.&RUNID._prePostCodes;
		        set _prePostCodes;
		        run;
				data DPLocal.&RUNID._allDeliveries;
				set _Deliveries;
				run;
			%end;

			%if %eval(&type.=5 | &type.=6) %then %do; 
				data DPLocal.&RUNID._dispensings;
		        set _dispensings;
		        run;
			%end;
	    %END;
	    %ELSE %DO;
			*Because of surveillance mode and the carried forward  variables from look <&PERIODIDSTART. that have not been 
	         calculated yet for the (&PERIODIDSTART.<=look<=&PERIODIDEND.)) - we will use a set method rather than append;

			%if %eval(&type ne 4) %then %do;
		        data DPLocal.&RUNID._mstr;
		        set DPLocal.&RUNID._mstr
		            _PtsMasterList;
		        run;   
			%end; 

			%if %eval(&type eq 4) %then %do; 
				proc append base=DPLocal.&RUNID._mstr_preg
			     			data=_PtsMasterList force;
				run;
				proc append base=DPLocal.&RUNID._sec 
		                    data=_PtsMasterList2 force;
		        run;
				%if "&t4_nopreggroup." = "Y" %then %do;
					proc append base=DPLocal.&RUNID._mstr_nopreg 
			                    data=_ControlGroup force;
				%end;
		        run;
				proc append base=DPLocal.&RUNID._prePostCodes 
		                    data=_prePostCodes force;
		        run;
				proc append base=DPLocal.&RUNID._allDeliveries 
		                    data=_Deliveries force;
		        run;
			%end;

			%if %eval(&type.=5 | &type.=6) %then %do;
				proc append base=DPLocal.&RUNID._dispensings
	                    data=_dispensings force;
	        	run;
			%end;
	    %END;

		%if %eval(&type ne 4) %then %do; 

			%if &censor_dth.=1 %then %do;
				*If censoring is used, truncate enrollment at death date (for denominators/attrition);
				data dplocal.enr_group&GROUP.;
				set dplocal.enr_&enrollmentnum.;
					if EnrEpisodeCensoredAtDeath = 1 and DeathDt <= Enr_End then do;
		       	  		Enr_End=DeathDt;
		        	end;
					if Enr_end < Enr_Start then delete;
				run;

				data _groupindex;
				set _groupindex;
					if EnrEpisodeCensoredAtDeath = 1 and . < DeathDt < adate then delete;
				run;
			%end;

			*Compute Attrition for this group;
	        %ms_attrition();

			proc datasets nowarn noprint lib=dplocal;
				delete enr_group&group.;
			quit;

		%end;

		* Reset loop macro variables;
		%let EXCL=;
		%let INCL=;
		%let CHARTRES=;

		* Save _GroupIndex for later processing;
		%if %eval(&concomitant.=1) %then %do;
			data gi&itgroup.;
				set _groupindex;
			run;
		%end;

		proc datasets nowarn noprint lib=work;
			delete _pov: _uniquepts: _age: _control: _deliveries _demo: _groupindex: _it: _next: _ptsmasterlist: _groupwash: _t&type.priorlooks;
		quit;

		/*Output loop runtime*/
		%ms_stoptimer(timestart=loopruntime);
		%ms_outputruntimes(timevar=loopruntime, step=%str(CIDA Loop), group=&itgroup., monitoringperiod=);

	%put LAST GROUP RAN WAS &ITGROUP. &group.;
	%PUT &ITGROUP. processing ended on %sysfunc(date(),date9.) %sysfunc(time(),tod8.);

	%end; /*group loop*/

	%mend forloop;

	%forloop;

	*Adjust enrollment for death for later processing;
	%if %sysfunc(exist(infolder.&type4file.)) ne 1 %then %do;
		%do e = 1 %to &numenrollgroup.;
		data dplocal.enr_&e.;
			set dplocal.enr_&e.;
				if EnrEpisodeCensoredAtDeath = 1 and DeathDt <= Enr_End then do;
			       	 	Enr_End=DeathDt;
			        end;
				if Enr_end < Enr_Start then delete;
		run;
		%end;
	%end;
	
	proc sort data=DPLocal.&RUNID._mstr&preg.;
	by Group PatId IndexDt;
	run;

	%if %eval(&type.=4) & "&t4_nopreggroup." = "Y" %then %do;
		proc sort data=DPLocal.&RUNID._mstr_nopreg;
		by Group PatId IndexDt;
		run;
	%end;

	*if data-driven query period, assign calendar based macro variables for reporting;
	%if "&datadrivenqueryperiod" = "Y" %then %do;
		%ms_assigncalmacrovars(date1=&MinExpPeriodStartDt., date2=&MaxExpPeriodEndDt.);
	%end;

	/********************************************************************************************************/
	/* Freeze Patients Data - Will save A/Y patients now. Matched patients saved after covariate_adjustment */
	/********************************************************************************************************/
	%macro wrapper;    
		%IF %str("&FREEZEDATA.")=%str("y") | %str("&FREEZEDATA.")=%str("a") %THEN %DO;

			%ms_starttimer(timestart=freezeruntime);

			%if "&t4_nopreggroup." = "N" %then %do; 
				proc sort nodupkey data=DPLocal.&RUNID._mstr&preg. 
					/*excludes never-exposed cohorts if "Y" */
					%if %str("&FREEZEDATA.")=%str("y") %then (WHERE = (INDEX(UPCASE(GROUP), "_NVREXP") = 0)) ;
					out=_PtsList(keep=Patid);
					by Patid;
				run;
			%end;
			%else %do;
				data _PtsList;
				set DPLocal.&RUNID._allDeliveries (keep=Patid)
					DPLocal.&RUNID._mstr_nopreg (keep=Patid);
				run;
				proc sort nodupkey data=_PtsList;
				by Patid;
				run;
			%end;
	       %isdata(dataset=_PtsList);
	        %if %eval(&nobs.>0) %then %do;            
	            %IF %STR(&ENRTABLE.) ne %STR() or %LENGTH(&ENRTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&ENRTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._enrtable);%END;            
	            %IF %STR(&DEMTABLE.) ne %STR() or %LENGTH(&DEMTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DEMTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._demtable);%END;            
	            %IF %STR(&DISTABLE.) ne %STR() or %LENGTH(&DISTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DISTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._distable);%END;            
	            %IF %STR(&DIATABLE.) ne %STR() or %LENGTH(&DIATABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DIATABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._diatable);%END;           
	            %IF %STR(&PROCTABLE.) ne %STR() or %LENGTH(&PROCTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&PROCTABLE,PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._proctable);%END;            
	            %IF %STR(&ENCTABLE.) ne %STR() or %LENGTH(&ENCTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&ENCTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._enctable);%END;           
	            %IF %STR(&LABTABLE.) ne %STR() or %LENGTH(&LABTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&LABTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._labtable);%END;           
	            %IF %STR(&VITTABLE.) ne %STR() or %LENGTH(&VITTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&VITTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._vittable);%END;            
	            %IF %STR(&DEATHTABLE.) ne %STR() or %LENGTH(&DEATHTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DEATHTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._deathtable);%END;
	            %IF %STR(&MILTABLE.) ne %STR() or %LENGTH(&MILTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&MILTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._miltable);%END;

	            *This datalib will point to the subset of PatId data generated by Freeze data;
	            %let datalib_enr=dplocal.&runid._enrtable;
	            %let datalib_dem=dplocal.&runid._demtable;
	            %let datalib_dis=dplocal.&runid._distable;
	            %let datalib_dia=dplocal.&runid._diatable;
	            %let datalib_proc=dplocal.&runid._proctable;
	            %let datalib_enc=dplocal.&runid._enctable;
	            %let datalib_lab=dplocal.&runid._labtable;
	            %let datalib_vit=dplocal.&runid._vittable;
	            %let datalib_death=dplocal.&runid._deathtable;
	            %let datalib_mil=dplocal.&runid._miltable;
	        %end;

			/*Output freezedata runtime*/
			%ms_stoptimer(timestart=freezeruntime);
			%ms_outputruntimes(timevar=freezeruntime, step=%str(Freeze Data), group=, monitoringperiod=);

	    %END;
	%mend wrapper;
	%wrapper;
	
	/********************/
	/* Switching module */
	/********************/
	%if %eval(&type. eq 6) %then %do;
		%ms_switching();
	%end;	

	/******************************/
	/* Secondary Episode Analysis */
	/*	-Multiple Events		  */
	/*	-Overlap				  */
	/*	-Concomitant Use Episodes */
	/******************************/
	%if %eval(&multievent.=1 | &overlap.=1) %then %do;
		%ms_evalsecondaryepi();
		%ms_evalsecondarytables();
	%end;

	%if %eval(&concomitant.=1) %then %do;
		%ms_concomitantuse();
	%end;

	/************************/
	/* Mother-Infant module */
	/************************/
	%if %eval(&milinkage. =1) %then %do;
		%ms_createmicohorts();
	%end;	

	/*** Extract Covariates and Produce Baseline table ***/
		
		*CIDA Groups: Types 1-5;
		%if %eval(&type. ne 6) & %eval(&multievent.=0) & %eval(&overlap.)=0 & %eval(&concomitant.=0) %then %do;
		    %ms_CIDAcov(data=mstr&preg., outcohort=&preg., groupvar=group, groupdefdata=cohortfile);
		%end;

		*CIDA Groups: Type 4 control group;
		%if %eval(&type.=4) & "&t4_nopreggroup" = "Y" %then %do;
			%ms_CIDAcov(data=mstr_nopreg, outcohort=_nopreg, groupvar=group, groupdefdata=cohortfile);
		%end;

		*Concomitant episodes;
		%if %eval(&concomitant.=1) %then %do;
			%ms_CIDAcov(data=mstr_concomitance, outcohort=_concomitance, groupvar=analysisgrp, groupdefdata=infolder.&concfile.);
		%end;

		*Multiple event analysis;
		%if %eval(&multievent.=1) %then %do;
			%ms_CIDAcov(data=mstr_multevent, outcohort=_multevent, groupvar=analysisgrp, groupdefdata=secondaryinputfile);
		%end;

		*Overlap event analysis;
		%if %eval(&overlap.=1) %then %do;
			%ms_CIDAcov(data=mstr_overlap, outcohort=_overlap, groupvar=analysisgrp, groupdefdata=secondaryinputfile);
		%end;

		*Mother-infant analysis;
		%if %eval(&milinkage. =1) %then %do;
			%ms_CIDAcov(data=mstr_mi, outcohort=_mi, groupvar=group, groupdefdata=migroupnames);
		%end;

		/*****************************************/
		/** Extract Claims Files (for MFU/HDPS) **/
        /*****************************************/

		%if %eval(&type. ne 6) & %eval(&multievent.=0 & &overlap.=0 & &concomitant.=0) %then %do;
        %ms_createclaimsfiles;
		%end;

		/**Conduct MFU analysis*/
		%isdata(dataset=infolder.&mfufile.);
		%if %eval(&nobs.>0) %then %do;

			%ms_starttimer(timestart=mfuruntime);

			%ms_mfu(where=1, t3out=)
			%if %eval(&type.=3) %then %do;
        	%ms_mfu(where=analysiscohort=1, t3out=_an);
			%end;

			%ms_stoptimer(timestart=mfuruntime);
			%ms_outputruntimes(timevar=mfuruntime, step=%str(MFU Module), group=, monitoringperiod=);
		%end;

		/*****************************/
		/* Produce Aggregate Outputs */
		/*****************************/
		%if %eval(&type. = 1) | %eval(&type. = 2) %then %do;
		    %ms_cidatables();
		%end;
		%if %eval(&type. eq 3) %then %do; 
			%ms_cidatablest3();		
		%end;
		%if %eval(&type. eq 4) %then %do;
			%ms_cidatablest4();
		%end;	
		%if %eval(&type. eq 5) %then %do;
			%ms_cidatablest5();
		%end;	
		%if %eval(&type. eq 6) %then %do;
			%ms_cidatablest6();
		%end;	
		%if %eval(&concomitant.=1) %then %do;
			%ms_concomitantusetables();
		%end;

		*output censoring table;
		%if "&censor_output." = "Y" %then %do;
			%ms_createcensortable();
		%end;

		*ITS analysis;
		%if %eval(&ITSANALYSIS.=1) %then %do;
			%ms_its();
		%end;

	*delete undocumented vars from MSTR;
	data dplocal.&RUNID._mstr&preg.(drop=EndFollowUpType blackoutper);
		set dplocal.&RUNID._mstr&preg.;
		if _n_ = 1 then do;
			dsid = open("dplocal.&RUNID._mstr&preg.");
				if varnum(dsid,"EndFollowUpType") = 0 then EndFollowUpType ='';
				if varnum(dsid,"blackoutper") = 0 then blackoutper =.;
			rc= close(dsid);
		end;
		drop rc dsid;

		%if %eval(&type.=5) %then %do;
			drop FEventDt; 
		%end;
		%if %eval(&type.=6) %then %do;
			drop tte; 
		%end;
	run;
			
	proc datasets library=work nowarn nolist;
		delete &cohortcodes. drugs drugscomb ENR_MD30 LABEXTRACT DeathExtract meds &MONITORINGFILE. &inclusioncodes. Enr_:
			PreMeds ElixMeds covmeds PreDrugs Monitoringfileflm Premeds Ptslist Studynames _:;
	quit;

	/****************************************/
	/* Covariate Adjusted matching analysis */
	/****************************************/

	%if "&matchanalysis" = "Y" %then %do;
		proc printto log="&MSOC.&RUNID._covariateadjustment.log" new;
		run;

		%ms_covariateadjustment();

		proc printto log="&MSOC.&RUNID._cida.log";
	  	run;
	%end;

	proc datasets library=dplocal nowarn nolist;
		delete _: drugs lab: mil: death: meds: drug: denom: enr_:;
	quit;

	/****************************/
	/* Freeze Patients Data - M */
	/****************************/
	%macro wrapper();
		%if %str("&FREEZEDATA.")=%str("m") %then %do;

			%ms_starttimer(timestart=freezeruntime);

			data _PtsList(keep=patid);
				set
				%do look=%eval(&periodidstart.) %to %eval(&periodidend.);
				 dplocal.&runid._matched_&look.
				%end;
				;
				where missing(matchid)=0;
			run;

			%if %eval(&type.=4) %then %do;
				data _PtsList2(rename=linkedid=patid);
					set
					%do look=%eval(&periodidstart.) %to %eval(&periodidend.);
					 dplocal.&runid._matched_&look.
					%end;
					;
					where missing(matchid)=0;
					keep linkedid;
				run;

				data _ptslist;
					set _PtsList _PtsList2;
				run;
			%end;

			proc sort nodupkey data=_PtsList;
				by Patid;
			run;

		    %isdata(dataset=_PtsList);
		    %if %eval(&nobs.>0) %then %do;            
		        %IF %STR(&ENRTABLE.) ne %STR() or %LENGTH(&ENRTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&ENRTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._enrtable);%END;            
	            %IF %STR(&DEMTABLE.) ne %STR() or %LENGTH(&DEMTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DEMTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._demtable);%END;            
	            %IF %STR(&DISTABLE.) ne %STR() or %LENGTH(&DISTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DISTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._distable);%END;            
	            %IF %STR(&DIATABLE.) ne %STR() or %LENGTH(&DIATABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DIATABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._diatable);%END;           
	            %IF %STR(&PROCTABLE.) ne %STR() or %LENGTH(&PROCTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&PROCTABLE,PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._proctable);%END;            
	            %IF %STR(&ENCTABLE.) ne %STR() or %LENGTH(&ENCTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&ENCTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._enctable);%END;           
	            %IF %STR(&LABTABLE.) ne %STR() or %LENGTH(&LABTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&LABTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._labtable);%END;           
	            %IF %STR(&VITTABLE.) ne %STR() or %LENGTH(&VITTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&VITTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._vittable);%END;            
	            %IF %STR(&DEATHTABLE.) ne %STR() or %LENGTH(&DEATHTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&DEATHTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._deathtable);%END;
	            %IF %STR(&MILTABLE.) ne %STR() or %LENGTH(&MILTABLE.) > 0. %THEN %DO;%MS_FREEZEDATA(INFILE=indata.&MILTABLE, PATLISTINFILE=_PtsList,OUTFILE=DPLocal.&RUNID._miltable);%END;

	            *This datalib will point to the subset of PatId data generated by Freeze data;
	            %let datalib_enr=dplocal.&runid._enrtable;
	            %let datalib_dem=dplocal.&runid._demtable;
	            %let datalib_dis=dplocal.&runid._distable;
	            %let datalib_dia=dplocal.&runid._diatable;
	            %let datalib_proc=dplocal.&runid._proctable;
	            %let datalib_enc=dplocal.&runid._enctable;
	            %let datalib_lab=dplocal.&runid._labtable;
	            %let datalib_vit=dplocal.&runid._vittable;
	            %let datalib_death=dplocal.&runid._deathtable;
	            %let datalib_mil=dplocal.&runid._miltable;
			%end;

			/*Output freezedata runtime*/
			%ms_stoptimer(timestart=freezeruntime);
			%ms_outputruntimes(timevar=freezeruntime, step=%str(Freeze Data), group=, monitoringperiod=);
		%end;
	%mend wrapper;
	%wrapper();

	/*******************/
	/* Tree extraction */
	/*******************/
	%if "&analysis" = "tree" %then %do;
		%if %eval(&type.=2) | %eval(&type.=4) %then %do;
		%ms_treeextraction(data=dplocal.&runid._matched_&periodidstart.(where=(missing(matchid)=0 and covarnum=0)), grp=analysisgrp);
		%end;
		%if %eval(&type.=3) %then %do;
		%ms_treeextraction(data=dplocal.&runid._mstr, grp=group);
		%end;
	%end;

	* Reset global macro variables;
	%let COMBORUN=;

	*Create the number of groups to create cohorts (into NOBS);
	%ISDATA(dataset=infolder.&COHORTFILE.);

	/*Output QRP runtime*/
	%ms_stoptimer(timestart=qrpruntime);

	data msoc.&runid._runtimes;
		set msoc.&runid._runtimes;
		if step = 'QRP Runtime' then do;
			format start stop datetime21.2; 
			start=input("&qrpruntime.",best.); 
			stop=input("&STOP.",best.);   
			format Seconds $20.;
			Seconds=put(int(&stop.-&qrpruntime.),best.)||" s";
			enddate = "&qrpruntimeenddate."; 
			endtime = "&qrpruntimeendtime.";
			runtime = strip("&hours. h &minutes. m &seconds. s");
			drop start stop seconds;
		end;
	run;
 
	*Signature file;
	data signature;
	
		/*Request meta data*/
		ReqID="&REQID.";
		ProjID="&PROJID.";  
		WPType="&WPTYPE.";
		WPID="&WPID.";
		DPID="&DPID.";
		VerID="&VERID.";
		RunID="&RUNID.";
		MPVer="&QRPVer.";
		CustomCodeVersion="&CustomCodeVer.";

		format StartTime StopTime datetime21.2; 
		StartTime=input("&qrpruntime.",best.); 
		StopTime=input("&STOP.",best.);   
		format Seconds $20.;
		Seconds=put(int(&stop.-&qrpruntime.),best.)||" s";
		ExecutionTime="&hours. h &minutes. m &seconds. s";    

		/*Data Partner meta data*/
		DP="&DP.";
		format DPMINDATE DPMAXDATE date9.;
		DPMINDATE=&DP_MinDate.;
		DPMAXDATE=&DP_MaxDate.;
		PATIDEXCL="&PATIDEXCL.";
		ETLNUMBER="&ETL.";
		BYPASSCC="&ccbypass.";
		RUNTYPE="&SYSENV.";
		SASVERSION="&SYSVER.";
		SASVERSIONLONG="&SYSVLONG.";
		OSABBR="&SYSSCP.";
		OSNAME="&SYSSCPL.";
		NCPU="&SYSNCPU.";

		/*Run Parameters*/
		ScenarioCnt=strip("&NOBS.");
		PERIODIDSTART="&PERIODIDSTART.";
		PERIODIDEND="&PERIODIDEND.";
		ANALYSIS="&ANALYSIS.";
		MONITORINGFILE="&MONITORINGFILE.";
		USERSTRATA="&USERSTRATA";
		COMBOFILE="&COMBOFILE.";
		COHORTFILE="&COHORTFILE.";
		COHORTCODES="&COHORTCODES.";
		INCLUSIONCODES="&INCLUSIONCODES.";
		STOCKPILINGFILE="&STOCKPILINGFILE.";
		RUN_ENVELOPE = "&RUN_ENVELOPE.";
		FREEZEDATA="&FREEZEDATA.";
		ZIPFILE="&ZIPFILE.";
		LABSCODEMAP="&LABSCODEMAP.";
		DISTINDEX = "&DISTINDEX.";
		TYPE1FILE="&TYPE1FILE.";
		TYPE2FILE="&TYPE2FILE.";
		TYPE3FILE="&TYPE3FILE.";
		TYPE4FILE="&TYPE4FILE.";
		PREGDUR="&PREGDUR.";	
		TYPE5FILE="&TYPE5FILE.";
		TYPE6FILE="&TYPE6FILE.";
		METADATAFILE="&METADATAFILE.";
		SURVEILLANCEMODE = "&SURVEILLANCEMODE";
		COVARIATECODES="&COVARIATECODES.";
		UTILFILE="&UTILFILE.";
		COMORBFILE="&COMORBFILE.";
		DRUGCLASSFILE="&DRUGCLASSFILE.";
		PROFILE = "&PROFILE.";
		MFUFILE="&MFUFILE.";
		MULTEVENTFILE="&MULTEVENTFILE.";
		MULTEVENTFILE_ADHERE="&MULTEVENTFILE_ADHERE.";
		CONCFILE="&CONCFILE.";
		OVERLAPFILE="&OVERLAPFILE.";
		OVERLAPFILE_ADHERE="&OVERLAPFILE_ADHERE.";
		ITSFILE="&ITSFILE.";
		TREATMENTPATHWAYS="&TREATMENTPATHWAYS.";
		MICOHORTFILE="&MICOHORTFILE.";
		TREEFILE="&TREEFILE.";
		TREELOOKUP="&TREELOOKUP.";
		ICD10ICD9MAP="&ICD10ICD9MAP.";

		/*Matching parameters*/
		COMPARISON="&COMPARISON.";
		ANALYTICSUBGROUPS="&ANALYTICSUBGROUPS.";
        COVARIATES_CONSIDERED="&COVARIATES_CONSIDERED.";
	    COVARIATES_SELECTED="&COVARIATES_SELECTED.";
	    RANKING="&COVARIATES_SELECTED.";
	    ZERO_CELL_CORR="&COVARIATES_SELECTED.";
		%if "&runhdps" = "Y" %then %do;
	    HDVARSEL_INPUT="&HDVARSEL_INPUT.";
		%end;
		%else %do;
		HDVARSEL_INPUT="";
		%end;	
	    INDLEVEL="&INDLEVEL.";
	    PERCENTILES="&PERCENTILES.";
	    UNCONDITIONAL="&UNCONDITIONAL.";
	    DIAGNOSTICS="&DIAGNOSTICS.";

		output;
	run;

	proc transpose data=signature out=msoc.&runid._signature(rename=_NAME_=Var rename=COL1=VALUE);
	   var _ALL_;
	run;

	data msoc.&RUNID._signature;
		set msoc.&RUNID._signature;
		value = strip(value);
	run;

	%put NOTE: ********END OF MACRO: ms_cidanum v4.1********;

%mend ms_cidanum;
