****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_its.sas  
*
* Created (mm/dd/yyyy): 05/15/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:                                  
*   
*  Program inputs:  
*             	                        
*  Program outputs:    
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_its();

	%put =====> MACRO CALLED: ms_its v1.0; 

	*Determine whether to produce AgeGroup and/or Sex stratifications;
	data _t2its;
		set userstrata;
		if lowcase(tableID) in ("t2its") then output _t2its;
	run;

	%isdata(dataset=_t2its);
	%if %eval(&nobs.>0) %then %do;

		%let t2itsvars = ;
		proc sql noprint;
			select distinct lowcase(levelvars) into: t2itsvars separated by ' '
			from _t2its;
		quit;

		%let outputsex = N;
		%let outputagegroup = N;
		%if %index(&t2itsvars., sex) >0 %then %let outputsex = Y;
		%if %index(&t2itsvars., agegroup) >0 %then %let outputagegroup = Y;

		/*_mstr file to use*/
		%if %eval(&multievent.=1) %then %do;
			%let data = _mstr_multevent;
		%end;
		%if %eval(&overlap.=1) %then %do;
			%let data = _mstr_overlap;
		%end;

		/*Determine which cov vars to output*/
		%let comorbout= ;
		%let utilout = ;
		%isdata(dataset=infolder.&comorbfile.);
		%if %eval(&nobs>0) %then %let comorbout = Y;
		%isdata(dataset=infolder.&utilfile.);
		%if %eval(&nobs>0) %then %let utilout = Y;

		*all intervals;
		proc sql noprint;
			select distinct interval into: intervallist
			from infolder.&itsfile;
		quit;

		%macro createitsnumcounts(class=,level=);
			proc means data=_mstr_its nway noprint missing;
				var denom1 num1 denom2 num2
					%do num = 1 %to 2;
						sex_m_&num. sex_f_&num. sex_o_&num. 
							%do age = 1 %to %sysfunc(countw(&AGESTRAT.," "));
								AgeGroup_&age._&num.
							%end;
							%if "comorbout" = "Y" %then %do;
								COMORBIDSCORE_&num.
							%end;
							%if "&utilout" = "Y" %then %do;
								numav_&num. numoa_&num. numip_&num. numis_&num. numed_&num.
								numgeneric_&num. numclass_&num. numrx_&num. 
							%end;
							%if %eval(&numcovars. > 0 | &numcomb. > 0) %then %do;
					           	 %do j=1 %to &numcovars.;%str(covar&j._&num.)%end;
								%do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k._&num.)%end;
							%end;
					%end;
						;
			        class analysisgrp &interval. interval_date year &class.;
			        output out=_outits(drop=_:) sum=;
		        run;

				proc sort data=_outits; 
					by analysisgrp interval_date &class.;
				run;

		        %if %eval(&s=1) %then %do;
		            data _itscounts;
		            	set _outits;
		           		format level $3.;
		           	 	Level="&level.";
		            run;
		        %end;
		        %else %do;
		            data _itscounts;
		            set _itscounts
		                _outits(in=a);
		                if a then do;
							level="&level.";
						end;
		            run;
		        %end;
			%mend;

		/*Loop through each ITS analysis*/
		%isdata(dataset=infolder.&ITSFILE.);
		%let itsloop = &nobs.;
		%do r = 1 %to %eval(&itsloop.);

			/*Assign macro variables*/
			data _null_;
				set infolder.&ITSFILE.;
				if _n_ = &r. then do;
					call symputx("analysisgrp", analysisgrp);
					call symputx("interval", lowcase(interval));
					call symputx("its_enrollment", lowcase(its_enrollment));
					call symputx("interventiondate1", interventiondate1);
					call symputx("interventiondate2", interventiondate2);
					call symputx("adherenceID", adherenceID);
				end;
			run;
			%put Analysis Group = &analysisgrp.;

			data _null_;
				set secondaryinputfile;
				where upcase(analysisgrp) = upcase("&analysisgrp");
				call symputx("primary", primary);
			run;

			%if "&interval" = "quarter" %then %let function = qtr;
			%else %let function = &interval;

			%if %eval(&adherenceID>0) %then %do;
			%let id = _&adherenceID;
			%end;
			%else %let id = ;

			*creating age group dataset;
			data _null_;
				set infolder.&cohortfile.;
					where upcase(cohortgrp) = upcase("&primary.");

				if missing(agestrat) then do;
					call symputx("agestrat","00-01 02-04 05-09 10-14 15-18 19-21 22-44 45-64 65-74 75+");
				end;
				else do;
					call symputx("agestrat",agestrat);
				end;
			run;

			/*square table based in interval*/
			data _null_;
			    format startdate enddate date9.;
			    startdate=intnx('month',&startdate.,0,'begin');  *in case not at the start of month;
			    enddate=intnx('month',&enddate.,1,'begin')-1; *in case not at the end of month;

				/*Interval can be year, quarter, month, week*/
				numinterval = intck("&function",startdate,enddate)+1;
		
			    call symputx("StartYear",year(startdate));
			    call symputx("StartMonth",month(startdate));
			    call symputx("EndYear",year(enddate));
			    call symputx("EndMonth",month(enddate));
			    call symputx("numinterval",numinterval);

			    do i=1 to numinterval+1;
			        call symput("PER"||strip(put(i,best.)),put(intnx("&function",&startdate.,i-1,'begin'),best.));
			    end;
	   		run;
	   		%PUT &StartYear. &EndYear. &numinterval.;
	    
			data _intervals;
				set infolder.&itsfile.;
				if _n_ = &r.;

				%if "&outputagegroup" = "Y" %then %do;      
				format AgeGroup $9.;
			    numagegrp = countw("&AGESTRAT."," ") ;

				do c=1 to numagegrp;
				      	AgeGroup=scan("&AGESTRAT",c," ");
				      	AgeGroupNum=c;
				%end;
				
			   		do per=1 to &numinterval.;
			       		*create year month variables;
					    interval_date=intnx("&function",&startdate.,per-1,'begin');
					    &interval.=INTINDEX("&function", interval_date);
						year=year(interval_date);
			                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
						%if "&outputsex" = "Y" %then %do;                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
						   sex="F";output;
						   sex="M";output;
						   sex="O";output;
						%end;
						%else %do;
							output;
						%end;
					end;

				%if "&outputagegroup" = "Y" %then %do;        
				end;	
				%end;

				format interval_date date9.;

			   keep analysisgrp &interval. interval_date year 
					%if "&outputagegroup" = "Y" %then %do; AgeGroup AgeGroupNum %end;
					%if "&outputsex" = "Y" %then %do; sex %end; ;
		    run;

		  	proc sql noprint;
		    	create table _mstr_its as 
		    	select mstr.*
				  ,  	case when mstr.sex = 'M' then 1
						else 0
						end as Sex_M_1
				  , 	case when mstr.sex = 'F' then 1
						else 0
						end as Sex_F_1
				  , 	case when mstr.sex = 'O' then 1
						else 0
						end as Sex_O_1
					%do age = 1 %to %sysfunc(countw(&AGESTRAT.," "));
				  ,		case when mstr.AgeGroupNum= %eval(&age.) then 1
						else 0
						end as AgeGroup_&age._1
					%end;
					%if "&comorbout." = "Y" %then %do;
					,	COMORBIDSCORE as COMORBIDSCORE_1
					%end;
					%if "&utilout." = "Y" %then %do;
					,	numav as numav_1
					,	numoa as numoa_1
					,	numip as numip_1
					,	numis as numis_1
					,	numed as numed_1
					,	numgeneric as numgeneric_1
					,	numclass as numclass_1
					,	numrx as numrx_1
					%end;
					%if %eval(&numcovars. > 0 | &numcomb > 0) %then %do;
				    	%do j=1 %to &numcovars.;
							, covar&j. as covar&j._1
						%end;
						%do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);
						   , covar&k. as covar&k._1
						%end;
					%end;
		    	from DPLocal.&RUNID.&data. as mstr 
				where upcase(analysisgrp) = upcase("&analysisgrp")

				/*Determine if enrollment criteria met*/
				%if "&its_enrollment" = "y" %then %do;
					and mstr.enr_start <= &startdate. and mstr.enr_end >= &enddate.
				%end;
				;
			quit;

			data _mstr_its(rename=episode=denom1 rename=adherence&id.=num1 rename=indenominator=denom2 rename=adherence&id._truncated=num2);
			    set _mstr_its(in=a)
			        _intervals(in=b) end=eof;
			    	if a then do;
						/*Assign to correct interval - determine date at start of interval
							Necessary because when interval is a week, index dates at the begining of the year that are midweek need to be reassigned to prior yr*/
						date=intnx("&function.",indexdt,1,'begin')-1; *date at end of interval;
						numinterval = intck("&function.",&startdate.,date); /*number of intervals between startdate and indexdt*/
						interval_date = intnx("&function.",&startdate.,numinterval,'begin'); /*recalculated indexdt*/
						&interval.=&function.(interval_date);
						year=year(interval_date); 

			        	episode=1;
						if adherence&id._truncated = . then adherence&id._truncated = 0;
						if adherence&id. = . then adherence&id._truncated = 0;

						if indenominator = 1 then do;
							sex_m_2 = sex_m_1;
							sex_f_2 = sex_f_1;
							sex_o_2 = sex_o_1;
							%do age = 1 %to %sysfunc(countw(&AGESTRAT.," "));
							AgeGroup_&age._2 = AgeGroup_&age._1;
							%end;
							%if "&comorbout." = "Y" %then %do;
							COMORBIDSCORE_2 = COMORBIDSCORE_1;
							%end;
							%if "&utilout." = "Y" %then %do;
							numav_2 = numav_1;
							numoa_2 = numoa_1;
							numip_2 = numip_1;
							numis_2 = numis_1;
							numed_2 = numed_1;
							numgeneric_2 = numgeneric_1;
							numclass_2 = numclass_1;
							numrx_2 = numrx_1;
							%end;
							%if %eval(&numcovars. > 0 | &numcomb > 0) %then %do;
				           	 	%do j=1 %to &numcovars.;%str(covar&j._2=covar&j._1;)%end;
								%do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k._2=covar&k._1; )%end;
							%end;
						end;
			        end;
			    	else do;
			        	episode=0;  
						adherence&id. = 0;
						indenominator = 0;
						adherence&id._truncated = 0;
					end;

					if not a then do;
						%do num = 1 %to 2; /*Need numerators 1 and 2 (non-truncated and truncated)*/
							sex_m_&num. = 0;
							sex_f_&num. = 0;
							sex_o_&num. = 0;
							%do age = 1 %to %sysfunc(countw(&AGESTRAT.," "));
							AgeGroup_&age._&num. = 0;
							%end;
							%if "&comorbout." = "Y" %then %do;
							COMORBIDSCORE_&num. = 0;
							%end;
							%if "&utilout." = "Y" %then %do;
							numav_&num. = 0;
							numoa_&num. = 0;
							numip_&num. = 0;
							numis_&num. = 0;
							numed_&num. = 0;
							numgeneric_&num. = 0;
							numclass_&num. = 0;
							numrx_&num. = 0;
							%end;
							%if %eval(&numcovars. > 0 | &numcomb > 0) %then %do;
				           	 	%do j=1 %to &numcovars.;%str(covar&j._&num.=0;)%end;
								%do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k._&num.=0; )%end;
							%end;
						%end;
					end;
					if indenominator ne 1 then do; /*only reset _2 vars*/
						    adherence&id._truncated = 0;
							sex_m_2 = 0;
							sex_f_2 = 0;
							sex_o_2 = 0;
							%do age = 1 %to %sysfunc(countw(&AGESTRAT.," "));
							AgeGroup_&age._2 = 0;
							%end;
							%if "&comorbout." = "Y" %then %do;
							COMORBIDSCORE_2 = 0;
							%end;
							%if "&utilout." = "Y" %then %do;
							numav_2 = 0;
							numoa_2 = 0;
							numip_2 = 0;
							numis_2 = 0;
							numed_2 = 0;
							numgeneric_2 = 0;
							numclass_2 = 0;
							numrx_2 = 0;
							%end;
							%if %eval(&numcovars. > 0 | &numcomb. > 0) %then %do;
				           	 	%do j=1 %to &numcovars.; %str(covar&j._2=0;)%end;
								%do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k._2=0;)%end;
							%end;
					end;
					format interval_date date9.;
					
					*Assign covar 2 labels;
					%if %eval(&numcovars. > 0 | &numcomb. > 0) %then %do;
						call execute('data _mstr_its; set _mstr_its;');
				    	%do j=1 %to &numcovars.;
						 	if eof then call execute("label covar&j._2="||'"'||strip(vlabel(covar&j._1))||'";');
						%end;
						%do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);
						 	if eof then call execute("label covar&k._2="||'"'||strip(vlabel(covar&k._1))||'";');
						%end;
						call execute ('run;');
					%end;
		    run;

			%isdata(dataset=_t2its);   
			%if %eval(&nobs.>0) %then %do; 
				%do s = 1 %to %eval(&nobs.);
					data _null_;
						set _t2its;
						if _n_ = &s. then do;
							call symputx('levelid', levelid);
							call symputx('levelvars', levelvars);
						end;
					run;

					%createitsnumcounts(class=&levelvars.,level=&levelid.);

					proc datasets nowarn noprint lib=work;
						delete _outits;
					quit;

				%end;

				%if %eval(&r.=1) %then %do;
					data t2_its;
						set _itscounts;
					run;
				%end;
				%else %do;
					data t2_its;
						set t2_its
							_itscounts;
					run;
				%end;		

				proc datasets nowarn noprint lib=work;
					delete _mstr_its _intervals;
				quit;
			%end;

		%end; /*loop through Analyses*/

		*Save to MSOC;
		data msoc.&runid._t2_its(rename=interval_date=interval);
			retain analysisgrp level &intervallist. interval_date year
				%if "&outputsex" = "Y" %then %do; sex %end;
				%if "&outputagegroup" = "Y" %then %do; agegroup agegroupnum %end;;			
			set t2_its;
		run;

		proc datasets nowarn noprint lib=work;
			delete t2_its;
		quit;

	%end; /*ITS output exists in USERSTRATA*/

%put NOTE: ********END OF MACRO: ms_its v1.0********;

%mend ms_its;
